/*======================================================================*
 *  Module .............Btech.lib                                       *
 *  File ...............btech.h                                         *
 *  Author .............Karl Leeser                                     *
 *  Creation Date ......23 August 1997                                  *
 *                                                                      *
 *             Copyright (c) 1997 Barrett Technology, Inc.              *
 *                        139 Main Street                               *
 *                       Kendall/MIT Square                             *
 *                  Cambridge, MA  02142-1528  USA                      *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *                   
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  * 
 *  lost profits, or any incidental, special, or consequential damage.  *
 *                                                                      *
 *                        SUPPORTED COMPILERS                           *
 *                                                                      *
 *  This software has been compiled and tested with the following 'C'   *
 *  compilers:                                                          *
 *                                                                      *
 *  - Microsoft Visual C v1.52                                          *
 *                                                                      *
 *  ******************************************************************  *
 *                                                                      *
 *  NOTES:                                                              *
 *   This file is the header file for the Barrett Technology, Inc.      *
 *   motion control library.                                            *
 *                                                                      *
 *  REVISION HISTORY:                    								*
 *	980708 - dr changed SAFETY_BIT to 15                                *
 *                                                                      *
 *======================================================================*/ 

#include <stdio.h>
#include <stdlib.h> 
#include <conio.h>
#include <math.h>
#include <time.h> 
#include <string.h> 

#include "pcdsp.h"     
#include "syscfg.h"
#include "vars.h"
#include "ripple.h"
#include "globals.h"
/*======================================================================*
 * Define software version number:                                      *
 *======================================================================*/
#define SW_DATECODE 1998.0 

/*======================================================================*
 * Switch based on robot configuration:                                 *
 *======================================================================*/
#if N_AXES == 4
	#define COORDSYS_ARY {0,1,2,3}
	#define SLEEP_POINT_ARY {HOME0, HOME1, HOME2, HOME3}  
	#define SLEEP_ANGLE_ARY {HOMEANG0,HOMEANG1,HOMEANG2,HOMEANG3}
	#define N_AXIS_ZERO_ARY {0,0,0,0}      
	#define DAMPING_ARY {DampCoeffM0,DampCoeffM1,DampCoeffM2,DampCoeffM3}
	#define N_AXIS_AWAKEN_ARY {0,37000, -37000, 0}
#elif N_AXES == 7
	#define COORDSYS_ARY {0,1,2,3,4,5,6}
	#define SLEEP_POINT_ARY {HOME0, HOME1, HOME2, HOME3, HOME4, HOME5, HOME6}    
	#define SLEEP_ANGLE_ARY {HOMEANG0,HOMEANG1,HOMEANG2,HOMEANG3,HOMEANG4,HOMEANG5,HOMEANG6}
	#define N_AXIS_ZERO_ARY {0,0,0,0,0,0,0}     
	#define DAMPING_ARY {DampCoeffM0,DampCoeffM1,DampCoeffM2,DampCoeffM3,DampCoeffM4,DampCoeffM5,DampCoeffM6}
	#define N_AXIS_AWAKEN_ARY {0, 37000, -37000, -10000, 0, 0, 0}
#endif

#define AXIS_INVALID !(axis<N_AXES && axis>=0) 

/*======================================================================*
 * Define safety stuff:                                                 *
 *======================================================================*/
#define SAFETY_BIT  16

/*---------------------------------------------------------*
 * Define the template for structure of type "Axis".       *
 * Define the Robot entity as an array of individual Axis  *
 * information and set aside storage for N_AXES.           *
 *---------------------------------------------------------*/   
struct Axis{
	int 	mpos;	/* current motor position 			*/
	float 	ma;   	/* corresponding motor angle 		*/
	float 	ja;   	/* and joint angle 					*/
	float 	s;    	/* sine of the joint angle 			*/
	float 	c;    	/* cosine of the joint angle 		*/
	float 	jt;   	/* joint torque 					*/
	float 	mt;   	/* motor torque 					*/
	int 	mo;		/* marker pulse offset from mpos=0	*/
	int* 	tr;    	/* torque ripple array 				*/
	}Robot[N_AXES];
	 	
/*---------------------------------------------------------*
 * Define the corresponding robot pointer type.            *
 *---------------------------------------------------------*/      
typedef struct Axis* robottype; 

/*---------------------------------------------------------*
 * Define the template for structure of type "Element"     *
 * which will be used in linked-list fashion, holding data *
 * for an N-axis array of positions and a pointer to the   *
 * next such list.                                         *
 *---------------------------------------------------------*/
struct Element{
	double data[N_AXES];	/* motor position data array   */
	struct 	Element* next; 	/* pointer to next Element	   */
	};
	 	
/*---------------------------------------------------------*
 * Define the pointer to such a structure.                 *
 *---------------------------------------------------------*/       
typedef struct Element* listtype;   

/*======================================================================*
 * Define torque ripple constants                                       *
 *======================================================================*/
#define TR_ARY_LENGTH 4096 
#define TR_FILENAME "smth_tr.d"
#define TR_PATH "c:\\btech\\trdata\\"           //dr changed for tr comparison
#define TR_DEST "test_tr.d"
#define INDEX_WINDOW 50

    
/*======================================================================*
 * Define robot control mode abstractions:                              *
 *======================================================================*/
#define POSITION    0
#define TORQUE      1


/*======================================================================*
 * Define Jacobian constructs/abstractions:                             *
 *======================================================================*/
#define X 0
#define Y 1
#define Z 2    

#define CTS2RAD ((2*PI)/4096)  /* mnemonic to convert from motor position to radians */
#define RAD2CTS (1/CTS2RAD)

/*======================================================================*
 * Define Kinematic constants [inches]:                                 *
 *======================================================================*/
#define L3 23.0
#define L4 11.0
#define D3 1.875
#define D4 D3
#define Z_OFFSET 18.7 /* height of shoulder axis above floor */ 

#define J0_RANGE_POS	(141*PI/180)	/* rad */ 
#define J0_RANGE_NEG	-J0_RANGE_POS
#define J1_RANGE_POS	(110*PI/180)	/* rad */ 
#define J1_RANGE_NEG	-J1_RANGE_POS
#define J2_RANGE_POS	(157*PI/180)	/* rad */ 
#define J2_RANGE_NEG	-J2_RANGE_POS 
#define J3_RANGE_POS	(PI)		/* rad */ 
#define J3_RANGE_NEG	-(52*PI/180) 
#define J4_RANGE_POS	(PI)	/* rad */ 
#define J4_RANGE_NEG	-J4_RANGE_POS
#define J5_RANGE_POS	(HALFPI)	/* rad */ 
#define J5_RANGE_NEG	-J5_RANGE_POS
#define J6_RANGE_POS	(TWOPI*3/8)	/* rad */ 
#define J6_RANGE_NEG	-J6_RANGE_POS           


/*======================================================================*
 * Define gravity compensation constants                                *
 *======================================================================*/
/*  these values were determined by sampling 1000 times per experimental position
	the program was run 3 times and the values were averaged	*/
	

#if N_AXES==7 && HAND==FALSE
	#define A (float)-14170
	#define B (float)-41081
	#define C (float) 11373
	#define D (float) 207312   
    
#elif N_AXES==7 && HAND==TRUE  
	#define A (float)-16828
	#define B (float)-71636
	#define C (float) 22968
	#define D (float) 226712   

#elif N_AXES==4
	#define A (float)-14947
	#define B (float)-5565
	#define C (float)-12142
	#define D (float) 73550 

#endif  	   


#define N_AXES_GCOMP 4 /* Number of axes under gravity compensation */
#define N_AXIS_GCOMP_ZERO_ARY {0,0,0,0}    

#define TORQUE_SAMPLES 2000	/* number of sample to take when getting values for zero_g constants  */
/*======================================================================*
 * Define sleep routine stuff:                                          *
 *======================================================================*/  
#define HOME0 (0)
#define HOME1 (-36940)
#define HOME2 (36940)
#define HOME3 (-36714)
#define HOME4 (0)
#define HOME5 (0)
#define HOME6 (0) 

#define HOMEANG0 (0)
#define HOMEANG1 (-1.9655)
#define HOMEANG2 (0)
#define HOMEANG3 (3.0771)
#define HOMEANG4 (0)
#define HOMEANG5 (0)
#define HOMEANG6 (0)



#define MOTION_EPSILON 1
#define DELAY 3		/* delay time so that a nonzero velocity is reached */      
#define DampCoeffM0 1 	/* motor coord damping coeff */   
#define DampCoeffM1 1 
#define DampCoeffM2 1 
#define DampCoeffM3 1 
#define DampCoeffM4 1 
#define DampCoeffM5 1 
#define DampCoeffM6 1  

#define SLEEP_SCALE_FACTOR 0.85 /* Fraction of full g-comp used during sleep() */
#define ZEROAXIS_SAMPLES 20	/* number of samples for finding repeatablitiy of awakenrobot*/

/*==========================================================*
 * GENERAL UTILITY FUNCTIONS:                               *
 *==========================================================*/ 	
void _error(char* s);
double getTime(void); 
double getLoopTime(void);
int getAutoRangedPosition(int);
void setMotorTorque(int, int);
int getMotorTorque(int);
void setMarkerOffset(robottype, int[N_AXES]);
float extractRobotElement(robottype, int, char*);
void displayRobot(robottype);  
int manMarkerFoundOffset(int);   
char MarkerFound(int);
int MarkerFoundOffset(int);
char manMarkerFound(int);  
void timeDelay(int);

/*==========================================================*
 * GLOBAL ACCESS FUNCTIONS:                                 *
 *==========================================================*/
void setTorqueRippleMode(char);
char getTorqueRippleMode(void); 

/*==========================================================*
 * GENERAL ROBOT FUNCTIONS:                                 *
 *==========================================================*/
void initRobot(void); 
void deinitRobot(void);
void killRobot(void);
void enableRobot(int);
void disableRobot(void);
void makeRobot(robottype);
char isEmptyRobot(robottype);
void destroyRobot(robottype);
void updateRobot(robottype);
void demojoints(robottype); 


listtype makePointList(void);
listtype makePointListElement(double*); 
char isEmptyPointList(listtype); 
void destroyPointList(listtype);  
listtype addPointListElement(listtype, double*); 
double* extractPointListData(listtype, int);
void printPointList(listtype);

listtype point_list;


/*==========================================================*
 * POSITION CONTROL FUNCTIONS:                              *
 *==========================================================*/    
void commandMove(double[N_AXES]);
void commandMoveJoint(double[N_AXES]);
void setupMove(double, double); 
void convertJointAngle2MotorPosition(double*);
void compensateTorqueRipple(robottype);
void compensateTorqueRipple2(robottype, int, int[COEFFICIENTS]); 

void coordinatedMove(listtype);
listtype teachMove(listtype, float, robottype); 
listtype teachMoveManual(listtype, robottype);


int loadTrajectoryFile(char*); 
int saveTrajectoryFile(char*);
void promptPlayback(double*, double*);

 
/*==========================================================*
 * TORQUE CONTROL FUNCTIONS:                                *
 *==========================================================*/
void actuateRobot(robottype, float, int);  
void compensateGravity(robottype);
void setJointTorque(robottype, float[N_AXES]);
void addJointTorque(robottype, float[N_AXES]); 
void calibrateZeroG(robottype);

/*==========================================================*
 * KINEMATIC FUNCTIONS:                                     *
 *==========================================================*/
void getCartesianPosition(robottype, float[3]);

/*==========================================================*
 * LIBRARY FUNCTIONS:                                       *
 *==========================================================*/
void _makeEmptyTorqueRippleAry(int**);  
void _makeTorqueRippleAry(int**, FILE*, char[12]); 
int _getSimulatedPosition(int);

/*==========================================================*
 * AWAKEN/SLEEP FUNCTIONS:  	                            *
 *==========================================================*/    
void awakenRobot(robottype);
void zeroAxisRepeatability(robottype);
char isJointLimitReached(int);
void Awaken1(void);
void Sleep1(void);     
void Sleep2(robottype);
char StillMoving(float []);
char StillMoving2(void);
void Sleep4(void);
#define JOINT_LIMIT_POSITION_ERROR 300

/*==========================================================*
 * OLD STYLE, ARRAY-BASED FUNCTIONS:                        *
 *==========================================================*/
float getMotorAngle(int);
void setTorques(float[], float[], float);  
void getAngles(float[], float[], int[]); 
void getGravityCompTorque(float[], float[], float[]);
void getTrig(float[], float[], float[]);



