/*======================================================================*
 *  Module .............Btech.lib                                       *
 *  File ...............btech.c                                         *
 *  Author .............Karl Leeser                                     *
 *  Creation Date ......23 August 1997                                  *
 *                                                                      *
 *             Copyright (c) 1997 Barrett Technology, Inc.              *
 *                        139 Main Street                               *
 *                       Kendall/MIT Square                             *
 *                  Cambridge, MA  02142-1528  USA                      *
 *                                                                      *
 *                        All rights reserved.                          *
 *                                                                      *
 *  ******************************************************************  *
 *                            DISCLAIMER                                *
 *                                                                      *
 *  This software and related documentation are provided to you on      *
 *  an as is basis and without warranty of any kind.  No warranties,    *
 *  express or implied, including, without limitation, any warranties   *
 *  of merchantability or fitness for a particular purpose are being    *
 *  provided by Barrett Technology, Inc.  In no event shall Barrett     *
 *  Technology, Inc. be liable for any lost development expenses, lost  *
 *  lost profits, or any incidental, special, or consequential damage.  *
 *                                                                      *
 *                        SUPPORTED COMPILERS                           *                
 *                                                                      *
 *  This software has been compiled and tested with the following 'C'   *
 *  compilers:                                                          *
 *                                                                      *
 *  - Microsoft Visual C v1.52                                          *
 *                                                                      *
 *  ******************************************************************  *
 *                                                                      *
 *  NOTES:                                                              *
 *   This file contains the function definitions for the Barrett        *
 *   Technology, Inc. motion control library.                           *
 *                                                                      *
 *  REVISION HISTORY:                                                   *
 *  980708 - updated markerfound() to use index- also                   *
 *  980801 - fixed error in actuateRobot JG                             *
 *======================================================================*/

#include "btech.h"
char COMPENSATE_TR = FALSE;
int list_length = 0;

 
/*==========================================================*
 * GENERAL UTILITY FUNCTIONS:                               *
 *==========================================================*/ 	

/*--------------------------------------------------------------*
 * _error()														*
 * 	                											* 
 * Prints outmessage and deinitializes robot					*
 * supplied robot structure.                                    *
 * 																*
 * Arguments:	char* s: 	- message to be printed				*
 *																*
 *--------------------------------------------------------------*/

void _error(char* s)
{
    printf("%s\n",s);
	deinitRobot();
	exit(1);
} 


double getTime(void)
{
    int time;

    time=get_dsp_time();
    
    printf("DSP time=%d\n",time); 
/*    
int FNTYPE init_timer(int channel, int mode);
int FNTYPE set_timer(int channel, unsigned t);
int FNTYPE get_timer(int channel, unsigned PTRTYPE * t) ;
unsigned FNTYPE get_dsp_time(void);
*/ 
  
    return((double)time);
}

/* return loop time in seconds */	 
double getLoopTime(void){
	double time=0,dt;
	static double timeOld=0;
	static char FirstTime=TRUE;
	
	/*----------------------------------------------------------*
 	 * Prevent potential large dt by returning a false, but  	*  
 	 * reasonable value the first time.                         *
 	 *----------------------------------------------------------*/
	if(FirstTime)
	{  
		dt=0.001;
		timeOld=getTime();
		FirstTime=FALSE;
	} 
	/*----------------------------------------------------------*
 	 * Otherwise, calculate the difference between present and	*  
 	 * past calls to getLoopTime().                             *
 	 *----------------------------------------------------------*/
	else{
		time=getTime();
		dt=time-timeOld;
		timeOld=time;
		} 
	return(dt);
	}


/*----------------------------------------------------------*
 * getAutoRangedPosition() returns the position of the axis	*  
 * between 0 and 4096			                            *
 *----------------------------------------------------------*/

int getAutoRangedPosition(int axis){
	double position;
	get_position(axis, &position);
	position=fmod(position,4096);
	if (position<0.0) position+=4096;
	return((int)position);
	}

/*----------------------------------------------------------*
 * setMotorTorque() sets the axis specified to the torque 	*  
 * specified.  It also limits the torque command to within  *
 * +/- 32767.												*
 *----------------------------------------------------------*/

void setMotorTorque( int axis, int MotorTorqueCommand){
     if(MotorTorqueCommand<-32767) MotorTorqueCommand=-32767;
     else if(MotorTorqueCommand>32767) MotorTorqueCommand=32767;
     set_dac_output(axis,MotorTorqueCommand);        /* +32767/-32768=+/-10V */
     }


/*----------------------------------------------------------*
 * getMotorTorque() returns the torque of the axis			*  
 * specified.					                            *
 *----------------------------------------------------------*/

int getMotorTorque( int axis){
	int torque, msg;

	if ((msg=get_dac_output(axis, &torque)) !=DSP_OK){
		printf("Error. get_dac_ouput: %d\n", msg);
		getchar();
	}
	return(torque);
} 


/*----------------------------------------------------------*
 * setMarkerOffset() sets the robot.mo (marker offsets) to	*  
 * the values specified in the markerOffsets array.         *
 *----------------------------------------------------------*/

void setMarkerOffset(robottype robot, int markerOffsets[N_AXES]){
	int axis;
    
    for(axis=0;axis<N_AXES;axis++){
    	robot->mo = markerOffsets[axis];
    	robot++;
    	}
	}
	

/*--------------------------------------------------------------*
 * extractRobotElement()											*
 * 	                											* 
 * Returns the specified element of the specified axis in the   *
 * supplied robot structure.                                    *
 * 																*
 * Arguments:	robottype robot -- pointer to robot structure	*
 *				int axis		-- axis number					*
 *				char* var		-- element (variable) desired	*
 *																*
 * Valid values for var are:									*
 * "mpos"	for		motor position								*
 * "ma"     for     motor angle									*
 * "ja"     for     joint angle									*
 * "s"      for     sin(ja)										*
 * "c"      for     cos(ja)										*
 * "jt"     for     joint torque								*
 * "mt"     for     motor torque								*
 * "mo"     for     marker offset position						*
 *                  											*
 * Usage:	foo = extractRobotElement(Robot,3,"mpos");					*
 *--------------------------------------------------------------*/
float extractRobotElement(robottype robot, int axis, char* var){
 	 
 	if(AXIS_INVALID) 
    	_error("Out of range argument axis in function extractRobotElement()");
 	/*----------------------------------------------------------*
 	 * Increment the pointer to the correct axis				*
 	 *----------------------------------------------------------*/
 	robot+=axis;

 	/*----------------------------------------------------------*
 	 * Compare var with the valid strings.  If there is a		* 
 	 * return the appropriate element.							*
 	 *----------------------------------------------------------*/      	
  	if		(strcmp(var,"mpos")==0) return((float)robot->mpos);
    else if	(strcmp(var,"ma")==0)  	return(robot->ma);
    else if	(strcmp(var,"ja")==0)  	return(robot->ja);
    else if	(strcmp(var,"s")==0)  	return(robot->s);
    else if	(strcmp(var,"c")==0)  	return(robot->c);
    else if	(strcmp(var,"jt")==0)  	return(robot->jt);
    else if	(strcmp(var,"mt")==0)  	return(robot->mt);
    else if	(strcmp(var,"mo")==0)  	return((float)robot->mo);
    
    else _error("Unrecognized argument in function extractRobotElement().");
	} 


/*--------------------------------------------------------------*
 * displayRobot()												*
 * 	                											* 
 * Returns the values stored in the robot structure			    *
 * supplied robot structure.                                    *
 * 																*
 * Arguments:	robottype robot -- pointer to robot structure	*
 *				int axis		-- axis number					*
 *				char* var		-- element (variable) desired	*
 *																*
 * Values returned are:											*
 * "mpos"	for		motor position								*
 * "ma"     for     motor angle									*
 * "ja"     for     joint angle									*
 * "s"      for     sin(ja)										*
 * "c"      for     cos(ja)										*
 * "jt"     for     joint torque								*
 * "mt"     for     motor torque								*
 * "mo"     for     marker offset position						*
 *                  											*
 * 																*
 *--------------------------------------------------------------*/

void displayRobot(robottype robot){
	int axis;
	
	for(axis=0;axis<N_AXES;axis++){     
		printf("Axis %d:",axis);
		printf("\tmpos\t=\t%d\n",robot->mpos);
		printf("\tma\t=\t%f\n",robot->ma);
		printf("\tja\t=\t%f\n",robot->ja);
		printf("\ts\t=\t%f\n",robot->s);
		printf("\tc\t=\t%f\n",robot->c);
		printf("\tjt\t=\t%f\n",robot->jt);
		printf("\tmt\t=\t%f\n",robot->mt);
		printf("\tmo\t=\t%d\n",robot->mo);
		printf("\tTR data not displayed.\n");
                                               
	    robot++;
		}
	}
	
/*==========================================================*
 * GLOBAL ACCESS FUNCTIONS:                                 *
 *==========================================================*/ 
/*--------------------------------------------------------------*
 * setTorqueRippleflag													*
 * 	                											* 
 * Sets the state of the torque ripple compensation global		*
 * variable, COMPENSATE_TR.	 As presently configured, the		*  
 * default state is COMPENSATE_TR=FALSE.						*
 * 																*
 * Arguments:	char flag		-- flag boolean					*
 *                  											*
 * Usage:	setTorqueRippleflag(TRUE);							*   
 *			setTorqueRippleflag(FALSE);							*
 *--------------------------------------------------------------*/
void setTorqueRippleMode(char state){
     COMPENSATE_TR = state;
     }

char getTorqueRippleMode(void){
     return(COMPENSATE_TR);
     }
	
/*==========================================================*
 * GENERAL ROBOT FUNCTIONS:                                 *
 *==========================================================*/
	
void initRobot(void){
	int axis;
	char buffer[MAX_ERROR_LEN];
	int e=dsp_init(PCDSP_BASE);
	int fp1_temp[10], fp2_temp[10];
	

	int fp0[10]={(int)(6000),20,(int)(19000),0,0,32767,0,32767,-5,0};
	int fp12[10]={(int)(6000),20,(int)(19000),0,0,32767,0,32767,-5,0};
	int fp3[10]={(int)(6000),20,(int)(19000),0,0,32767,0,32767,-5,0};
	int fp45[10]={(int)(500),2,4000,0,0,32767,0,9500,-5,0};
	int fp6[10]={(int)(900),2,7000,0,0,32767,0,3000,-5,0};   


	
 	if(HW_DATECODE>SW_DATECODE){
 	  	_error("Robot hardware/software version mismatch.  Exiting...");
 	}

	printf("Initializing...\n");
	if(e!= DSP_OK){
		printf("Can't initialize: %s\n",error_msg(e,buffer));
		_error("Exiting...");
	}

  for(axis=0;axis<N_AXES;axis++){
		set_conversion(axis, 1,1);              /* no conversion */
    	set_positive_limit(axis,NO_EVENT);             /*HW limit*/
		set_negative_limit(axis,NO_EVENT);
		set_positive_sw_limit(axis,HIGH_LIM,NO_EVENT); /*SW limit*/
		set_negative_sw_limit(axis,LOW_LIM,NO_EVENT);
		controller_idle(axis);
		set_dac_output(axis,0);
		set_in_position(axis,WINDOW);
		}
	
	// DR set ports 0 and 1 to inputs for index pulses
	init_io(0, IO_INPUT);
	init_io(1, IO_INPUT);	
	
	/*----------------------------------------------------------*
	 * Bit 0 of port 2 will be used to engage the Btech safety  *
	 * circuit. Note that we only reserve one bit, yet MEI      *
	 * requires that all bits in a port be configured the same. *
	 * Therefore, configure port 1 to be all outputs.           *
	 *----------------------------------------------------------*/
	init_io(2,IO_OUTPUT);   /* check for errors on this config. */
    set_bit(SAFETY_BIT);
    	
    
    /*  sets the filter parameters  */
    set_filter(0,fp0);
	set_filter(1,fp12);
	set_filter(2,fp12);
	set_filter(3,fp3);
	set_filter(4,fp45);
	set_filter(5,fp45);
	set_filter(6,fp6);
	
	get_filter(1, fp1_temp);	
	get_filter(2, fp2_temp);	
	
	printf("Initialization successful.\n");
	printf("Reset the Control Cabinet and press any key to continue\n");
	getch();	
} 

void deinitRobot(void)
{ /* Idle motors */
	int axis;
	int coefficients[COEFFICIENTS];

	printf("\nIdling motors...");
	for(axis=0;axis<N_AXES;axis++)
	{
		set_dac_output(axis, 0);
		controller_idle(axis);
		get_filter(axis, coefficients);
		coefficients[DF_OFFSET]=0;
		set_filter(axis,coefficients); 
	}
	printf("Motors idled.\n");

	//dr added to kill cabinet on dinitialization
    reset_bit(SAFETY_BIT);

}

void killRobot(void)
{
    reset_bit(SAFETY_BIT);
}

void enableRobot(int mode){
     int axis;

     switch(mode){
          case POSITION:
               for(axis=0;axis<N_AXES;axis++)
               {
                    controller_run(axis);
                    controller_run(axis);
                    set_amp_enable(axis,TRUE);
               }
               break;
          case TORQUE:
               for(axis=0;axis<N_AXES;axis++){
                    controller_idle(axis);
                    controller_idle(axis);
                    set_amp_enable(axis,TRUE);
                    }
               break;
          default:
               _error("Error...Out of range argument mode in enableRobot().\n");
               break;
          }
     }

void disableRobot(void){ 
	int axis;
	
	printf("Disabling Motors...\n");
	/* Disable amp and controller */
	for(axis=0;axis<N_AXES;axis++)
	{ 
		controller_idle(axis);
		set_amp_enable(axis,FALSE);
	}
} 

	
/* history of function */
/* 980708 -dr added TR_PATH to file name and increased length from 12 to 100 */
void makeRobot(robottype robot){
    int axis; 
    FILE* fp;
	char filename[100];
    
   	for(axis=0;axis<N_AXES;axis++)
   	{
		robot->mpos=0;  
		robot->ma=0;
		robot->ja=0;
		robot->s=0;
		robot->c=0;
		robot->jt=0;
		robot->mt=0;
		robot->mo=0; /* Just zero marker offset for now */ 
        
 
        /*------------------------------------------------------*
 		 * If torque ripple compensation is specified, then     *
 		 * load the torque ripple data from file.  Otherwise,   *
 		 * initialize the torque ripple arrays with an empty    *
 		 * array (a zero array, 4096 elements long).            *
 		 *------------------------------------------------------*/
        /* use the following code when possible to use filename as variable */          
        /* do not do tr compensation for axis 6 */
        if(COMPENSATE_TR && !(axis==6))
        {
        	sprintf(filename, "%s%s%d", TR_PATH, TR_FILENAME, axis);
		}
		
		/* do not do tr compensation for axis 6 */
		if(COMPENSATE_TR && !(axis==6)) _makeTorqueRippleAry(&(robot->tr), fp, filename); 
	    else _makeEmptyTorqueRippleAry(&(robot->tr));
        robot++;  /* increment the robot ptr to the next axis */
	}
} 

char isEmptyRobot(robottype robot){
    return((robot==NULL) ? TRUE : FALSE);
	}     
	
void destroyRobot(robottype robot){
    int axis; 
      
   	if(!isEmptyRobot(robot)){
   		for(axis=0;axis<N_AXES;axis++){ 
   	    	free(robot->tr);
   	    	robot++;
   			}
   		}
	}
 

/*----------------------------------------------------------*
 * () updates all of the parameters in the			 	  	*
 * the robot structure.										*
 *															*
 *	History:												*
 *	980709	-dr need to update joint and motor transforms   *
 * 			for the base and for wrst 						*
 *----------------------------------------------------------*/

void updateRobot(robottype robot){
	int axis;
	float ma1, ma2, ma4,ma5;
	double motorPosition;  

	for(axis=0;axis<N_AXES;axis++){
	
		/*------------------------------------------------------*
 		 * Get the motor position and angle.					*
 		 *------------------------------------------------------*/ 
 		get_position(axis, &motorPosition);    
		robot->ma = (float)(motorPosition*CTS2RAD);

		/*------------------------------------------------------*
 		 * Get the joint angles by scaling the motor angles     *
 		 * according to the following Motor-Joint space angle   *
 		 * transformations: 	         			    *
 		 *                                                      *
 		 *  | ja1 |     | -1/N1  0       0        0 | | ma1 |	*
 		 *  | ja2 |  =  | 0   1/2N2    -1/2N2     0 | | ma2 | 	*
 		 *  | ja3 |     | 0  -n3/2N2  -n3/2N2     0 | | ma3 | 	*
 		 *  | ja4 |     | 0      0       0      1/N4| | ma4 |	*
 		 *                                                   	*
 		 *------------------------------------------------------*/

		/*------------------------------------------------------*
 		 * Get the joint angles by scaling the motor angles     *
 		 * according to the following Motor-Joint space angle   *
 		 * transformations: 	         			    *
 		 *                                                      *
 		 *  | ja5 |     |  1/2N5       1/2N5    0   | | ma5 |	*
 		 *  | ja6 |  =  | -n6/2N5     n6/2N5    0   | | ma6 | 	*
 		 *  | ja7 |     |    0         0      -1/N7 | | ma7 | 	*
 		 *                                                   	*
 		 *------------------------------------------------------*/
 
 		
		switch(axis){
		    case 0:
		    	robot->ja = -((robot->ma)/N1);
		    	break;
		    case 1:
		    	ma1 = robot->ma; 
		    	break;
		    case 2:
				ma2 = robot->ma;
				robot--;
				robot->ja = (ma1)/(2 * N2) - (ma2)/(2*N2); 
				robot->s = (float)sin(robot->ja);
				robot->c = (float)cos(robot->ja); 
        		robot++;
		    	robot->ja = -(DIF3*(ma1)/(2*N2) + DIF3*(ma2)/(2*N2)); 
		    	break;
		    case 3:
		    	robot->ja = ((robot->ma)/N4); 
		    	break;
		    case 4:
		    	ma4=robot->ma;
		    	break;
		    case 5:
		    	ma5=robot->ma;
		    	robot--;
				robot->ja= 1/(2*N5)*ma4 +1/(2*N5)*ma5;
				robot->s = (float)sin(robot->ja);
				robot->c = (float)cos(robot->ja);
				robot++;
				robot->ja=-(DIF6)/(2*N5)*ma4 + (DIF6)/(2*N5)*ma5;
				break; 
		    case 6:
		    	robot->ja=-robot->ma/N7;
		    	break;
		    default:
		    	_error("Invalid axis call in updateRobot().");
			} 
		
		/*------------------------------------------------------*
 		 * Take the sines and cosines of the joint angles		*
 		 *------------------------------------------------------*/
		robot->s = (float)sin(robot->ja);
		robot->c = (float)cos(robot->ja); 
		
		/*------------------------------------------------------*
 		 * Reduce the motor position to its primary position,   * 
 		 *             0 <= motorPosition <= 4095               *
 		 *------------------------------------------------------*/
		motorPosition = fmod(motorPosition,4096);
		if (motorPosition<0.0) motorPosition+=4096;
		
		/*------------------------------------------------------*
 		 * Record this position in the robot structure.         *
 		 *------------------------------------------------------*/
		robot->mpos = (int)motorPosition;
			
		robot++;
		}
	}
 
/*****************************************************************
 * manMarkerFound will find the marker pulse using the index+    *
 * and index- manualy.  It will return TRUE when marker found    *
 *****************************************************************/

char manMarkerFound(int axis){
    int io=0;
    int bit_high=0, bit_low=1;
    double pos;

    printf("Move axis %d to find the marker pulse.\n",axis);

//   	init_io(0,IO_INPUT); // DR removed because when a port is set to input/output all other bits are cleared
//   	init_io(1,IO_INPUT);
    get_position(axis, &pos);

    while( !((bit_high=bit_on(axis))  && !(bit_low=bit_on(axis+8))) && !kbhit()){
    	pos=getAutoRangedPosition(axis);
		printf("Axis %d position %.1lf: Index+ =%d   Index- = \r",axis, pos,bit_high, bit_low);

		while(!axis_done(axis)){
			if(kbhit()) _error("Exiting...");
			}
		}

    set_position(axis,0);
  	return(TRUE);
}


/*****************************************************************
 * manMarkerFoundOffset will find the marker pulse using the     *
 * index+ and index- manually.  It will return the offset value  *
 *****************************************************************/

int manMarkerFoundOffset(int axis)
{
    int io=0;
    int bit_high=0, bit_low=1;
    int pos;

//dr modified to automatically search for index pulse
    printf("\nMove motor axis %d to find the marker pulse.\n",axis);
//    getch();
    
//   	init_io(0,IO_INPUT);  // DR removed because when a port is set to input/output all other bits are cleared
//   	init_io(1,IO_INPUT);
//	pos=getAutoRangedPosition(axis);
//	printf("Testing:	Start position = %d\n", pos);

    while( !( (bit_high=bit_on(axis))  && !(bit_low=bit_on(axis + 8)) ) && !kbhit() )
    {
    printf("\rbit_high=%d\tbit_low=%d", bit_high, bit_low);
	printf("\tCurrent position: %d", getAutoRangedPosition(axis));
    }
/* use this for debugging    	printf("\nbit_high=%d\tbit_low=%d", bit_high, bit_low);*/
    	pos=getAutoRangedPosition(axis);
/* use this for debugging		printf("\nAxis %d position %d: Index+ =%d   Index- = %d\r",axis, pos,bit_high, bit_low);*/

	printf("\nMarker %d = %d",axis, pos);
  	return((int)pos);
}
   



void promptPlayback(double* vel, double* accel)
{
	int key=0;
	
	printf("\n\nPlayback parameters: \t velocity \t= \t%.0lf\n \t\t\tacceleration \t= \t%.0lf\n\n",*vel,*accel);
	printf("Press <p> to change parameters, any other key to continue.\n");
	if(getch()=='p')
	{
		printf("What speed (%.0lf) ?\t\t",*vel);
		scanf(" %lf",vel);
		printf("What acceleration (%.0lf) ?\t",*accel);
		scanf(" %lf",accel);
	}
}
	

/*==========================================================*
 * POSITION CONTROL FUNCTIONS:                              *
 *==========================================================*/ 

void setupMove(double vel, double accel){
	int coordsys[N_AXES]=COORDSYS_ARY;
      
	map_axes(N_AXES,coordsys); 
	set_move_speed(vel);
	set_move_accel(accel);
}

void commandMove(double CommandPosition[]){
		start_point_list();
    	move_n(CommandPosition);
    	end_point_list();
    	start_motion();
   		}  
 	
 	/* jaAry is an array of joint angles in radians */
void commandMoveJoint(double jaAry[N_AXES])
{
    double mposAry[N_AXES]=N_AXIS_ZERO_ARY;  

    /* jaAry contains motor positions after this command */
    convertJointAngle2MotorPosition(jaAry);

	/*------------------------------------------------------*    
     * Since commandMove() takes an array of doubles,       * 
     * convert the floats to doubles.                       *
     *------------------------------------------------------*/

    mposAry[0]=jaAry[0]; 
    mposAry[1]=jaAry[1];
    mposAry[2]=jaAry[2];
    mposAry[3]=jaAry[3];
    
    if(N_AXES==7)
    {
    	mposAry[4]=jaAry[4];
    	mposAry[5]=jaAry[5];
    	mposAry[6]=jaAry[6];
    }    
    commandMove(mposAry);	
}



/*----------------------------------------------------------*
 *	demojoints(robottype) allows the user to specify what   *
 * 	joint angles to move to and then moves the robot to 	*
 *	the specified position.
 *----------------------------------------------------------*/


void demojoints(robottype robot){
    float mt[N_AXES]=N_AXIS_ZERO_ARY;
    int axis; 
    int	exit=FALSE, actuate=TRUE;
    int	key;
    char answer;
    double* ja_point;
    double ja[N_AXES];

   /*----------------------------------------------------------*
 	 * Enable closed-loop position control on the axis.  Most   *  
 	 * likely, it is already in this mode, so the call may be   *
 	 * redundant, but that's ok.                                *
 	 *----------------------------------------------------------*/
   
    setupMove(SLOW_VEL,SLOW_ACCEL);
	enableRobot(POSITION);
    
    while(!exit)
    {
		
		system("cls");
		printf("\nJoint limits are:");
		printf("\nJoint 1\tMin:\t%.3f\tMax:\t%.3f", J0_RANGE_NEG, J0_RANGE_POS);
		printf("\nJoint 2\tMin:\t%.3f\tMax:\t%.3f", J1_RANGE_NEG, J1_RANGE_POS);
		printf("\nJoint 3\tMin:\t%.3f\tMax:\t%.3f", J2_RANGE_NEG, J2_RANGE_POS);
		printf("\nJoint 4\tMin:\t%.3f\tMax:\t%.3f", J3_RANGE_NEG, J3_RANGE_POS);		
		if(N_AXES==7)
		{
			printf("\nJoint 5\tMin:\t%.3f\tMax:\t%.3f", J4_RANGE_NEG, J4_RANGE_POS);		
			printf("\nJoint 6\tMin:\t%.3f\tMax:\t%.3f", J5_RANGE_NEG, J5_RANGE_POS);		
			printf("\nJoint 7\tMin:\t%.3f\tMax:\t%.3f", J6_RANGE_NEG, J6_RANGE_POS);		
		}		
		
		printf("\nEnter angles for each joint in radians.");
		
		
		for(axis=0;axis<N_AXES;axis++)
		{
			printf("\nAxis %d = ", (axis+1));
		 	scanf("%lf", &(ja[axis]));
		}

		
		/* test joint angles for limits*/
		if(ja[0]>J0_RANGE_POS || ja[0]<J0_RANGE_NEG)
		{
			actuate=FALSE;
			printf("\nERROR: Joint 1 must be between %.3f and %.3f", J0_RANGE_POS, J0_RANGE_NEG);
		}
		if(ja[1]>J1_RANGE_POS || ja[1]<J1_RANGE_NEG)
		{
			actuate=FALSE;
			printf("\nERROR: Joint 2 must be between %.3f and %.3f", J1_RANGE_POS, J1_RANGE_NEG);
		}
		if(ja[2]>J2_RANGE_POS || ja[2]<J2_RANGE_NEG)
		{
			actuate=FALSE;
			printf("\nERROR: Joint 3 must be between %.3lf and %.3f", J2_RANGE_POS, J2_RANGE_NEG);
		}
		if(ja[3]>J3_RANGE_POS || ja[3]<J3_RANGE_NEG)
		{
			actuate=FALSE;
			printf("\nERROR: Joint 4 must be between %.3f and %.3f", J3_RANGE_POS, J3_RANGE_NEG);
		}
		
		if(N_AXES==7)
		{
			if(ja[4]>J4_RANGE_POS || ja[4]<J4_RANGE_NEG)
			{
				actuate=FALSE;
				printf("\nERROR: Joint 5 must be between %.3f and %.3f", J4_RANGE_POS, J4_RANGE_NEG);
			}
			if(ja[5]>J5_RANGE_POS || ja[5]<J5_RANGE_NEG)
			{
				actuate=FALSE;
				printf("\nERROR: Joint 6 must be between %.3f and %.3f", J5_RANGE_POS, J5_RANGE_NEG);
			}
			if(ja[6]>J6_RANGE_POS || ja[6]<J6_RANGE_NEG)
			{
				actuate=FALSE;
				printf("\nERROR: Joint 7 must be between %.3f and %.3f", J6_RANGE_POS, J6_RANGE_NEG);
			}
		}
		
		if(actuate==TRUE)
		{
			printf("\nAngles for each joint in radians are\n");
	   		for(axis=0;axis<N_AXES;axis++) printf("\n%.3f", ja[axis]);	
			printf("\nPress <space> to continue, any other key to null values specified");
			key=getch();
			if(key==' ') actuate=TRUE;
			else	actuate=FALSE;
		}
		
		
		ja_point=ja;
		if(actuate)
		{
			set_move_speed(SLOW_VEL);
			set_move_accel(SLOW_ACCEL);
			commandMoveJoint(ja_point);
			while (!all_done()){};
		}
		printf("\nPress 'y' to move to different angle, any other key to exit.");
		answer=getch();
		if(answer!='y') exit=TRUE;
		actuate=TRUE;
	}
}



/*--------------------------------------------------------------*
 * convert joint angles to motor angles, where -2pi < ja < 2pi. *
 *																*
 * 980709 -dr update motor transfomations						*
 *--------------------------------------------------------------*/

void convertJointAngle2MotorPosition(double* ja){ 
	double ma[N_AXES]=N_AXIS_ZERO_ARY;  

    /*------------------------------------------------------*
 	 * Get the arm motor angles by scaling the joint angles *
 	 * according to the following Joint-Motor space angle   *
 	 * transformations:            							*
	 *                                                      *
	 *  | ma1 |     |-N1     0      0        0 | | ja1 |    *
	 *  | ma2 |  =  | 0      N2   -N2/n3     0 | | ja2 |    *
	 *  | ma3 |     | 0     -N2   -N2/n3     0 | | ja3 |    *
	 *  | ma4 |     | 0      0      0        N4| | ja4 |    *
	 *                                                      *
	 *------------------------------------------------------*/ 
	
    /*------------------------------------------------------*
 	 * Get the wrist motor angles by scaling the joint      *
 	 * angles according to the following Joint-Motor space  *
 	 * angle transformations:            					*
	 *                                                      *
	 *  | ma5 |     |  N5   -N5/n6   0   | | ja5 |	    	*
	 *  | ma6 |  =  |  N5    N5/n6   0   | | ja6 |    		*
	 *  | ma7 |     |  0       0    -N7  | | ja7 |    		*
	 *                                                      *
	 *------------------------------------------------------*/ 


	ma[0] = -N1*ja[0];
	ma[1] =  N2*ja[1] - (N2/DIF3)*ja[2]; 
	ma[2] = -N2*ja[1] - (N2/DIF3)*ja[2]; 
	ma[3] =  N4*ja[3];

	
	if(N_AXES==7)
	{	
		ma[4]= N5*ja[4] - (N5/DIF6)*ja[5];
		ma[5]= N5*ja[4] + (N5/DIF6)*ja[5];  
		ma[6]=-N7*ja[6];
	}
	
	/*------------------------------------------------------*    
     * Next, convert motor angles to motor position.        *
     *------------------------------------------------------*/
	ja[0] = ma[0]*RAD2CTS;
	ja[1] = ma[1]*RAD2CTS;
	ja[2] = ma[2]*RAD2CTS;
	ja[3] = ma[3]*RAD2CTS;
	
	if(N_AXES==7){
		ja[4] = ma[4]*RAD2CTS;
		ja[5] = ma[5]*RAD2CTS;
		ja[6] = ma[6]*RAD2CTS;
	} 
} 

	
void compensateTorqueRipple(robottype robot)
{
    int axis, absolute_pos;
    int* fp, compensate_axes;

	/* no compensation for axis 7*/
	if(N_AXES==7) compensate_axes=N_AXES-1;
	else compensate_axes=N_AXES;
	    
    if(AXIS_INVALID) 
    	_error("Out of range argument axis in function applyTorqueRippleComp()");
    	
    if(COMPENSATE_TR)
    {
    	
    	for(axis=0; axis < compensate_axes; axis++)
    	{
    	    absolute_pos=robot->mpos - robot->mo;
    	    if (absolute_pos<0) absolute_pos+=4096;
    	    get_filter(axis,fp);
    		fp[DF_OFFSET]=robot->tr[absolute_pos];
			set_filter(axis,fp); 
			
			robot++;
		}
	}
}  


void compensateTorqueRipple2(robottype robot, int axis, int fp[COEFFICIENTS]){
    
    if(AXIS_INVALID) 
    	_error("Out of range argument axis in function applyTorqueRippleComp()"); 
    	
    if(COMPENSATE_TR){
		robot+=axis;
    	fp[DF_OFFSET]=robot->tr[robot->mpos-robot->mo];
		set_filter(axis,fp);
		} 
	} 
  
void coordinatedMove(listtype point_list)
{
	int element, err;
    listtype temp=point_list;
	int axis;
    double tmp_pos[N_AXES];

	arc_optimization(TRUE);
	/* put point list together */
	for(axis=0;axis<N_AXES;axis++)
	{
		err=get_position(axis, &tmp_pos[axis]);
	}
	
	err=start_point_list();
	err=move_n(tmp_pos);    
    for(element=0;element<list_length;element++)
    {                                
    	while (move_n(extractPointListData(point_list,element))!=DSP_OK) 
    		if(kbhit()) _error("Error...User interrupt.  Performing emergency kill...");
     	if(element==1) printf("Started Motion : %d\n", start_motion());                                          
     	if (element == list_length-1) printf("Finished filling DSP buffer : %d\n", end_point_list());
		temp=temp->next;
    }                          
         
	while(!all_done()) if(kbhit()) _error("Error...User interrupt.  Performing emergency kill...");
    printf("\n Actual Final Position");
    return;
} 
    

/* History of Function   */
/* 980708 - dr updated function to compensate for torque ripple while gathering points */

listtype teachMove(listtype point_list, float step, robottype Robot)
{
	int axis; 
	extern int list_length;
	double n_axis_point[N_AXES];
    float jtAry[N_AXES]=N_AXIS_ZERO_ARY;  
	clock_t t_init;
	char	key;

	printf("\nPreparing gravity compensation");	
	enableRobot(TORQUE);
	printf("\nPrepare to move arm through desired trajectory.\n"
			"Press 'r' when ready to start recording.\n"
			"When recording press any key to stop.\n");
			
	while(key!='r')
	{
		if(kbhit())	key=getch();
		setJointTorque(Robot,jtAry); 
		updateRobot(Robot);
		compensateGravity(Robot);
		actuateRobot(Robot,(float)1.0, N_AXES);
    }
	
	t_init=clock()/CLOCKS_PER_SEC;            
		
    for(list_length=0;!kbhit();list_length++)
    {
    	while( (float)(clock())/CLOCKS_PER_SEC < (float)(t_init)+( ((float)(list_length+1)) * step) )  
       	{
			setJointTorque(Robot,jtAry); 
			updateRobot(Robot);
			compensateGravity(Robot);
			actuateRobot(Robot, (float)1.0, N_AXES);
		}
       	
       	printf("\n\aPosition %d:",list_length);
		for(axis=0;axis<N_AXES;axis++)
		{ 
			/* Get the motor position. */
	 		get_position(axis,&n_axis_point[axis]); 
            printf("%.0lf,",n_axis_point[axis]);
		} 
		point_list=addPointListElement(point_list,n_axis_point);
	}
    
	enableRobot(POSITION);
	return(point_list);
}
	
listtype teachMoveManual(listtype point_list, robottype Robot)
{
	int axis;
	extern int list_length;
	double n_axis_point[N_AXES];
	char key;		
    float jtAry[N_AXES]=N_AXIS_ZERO_ARY;
    
    system("cls");
    
    enableRobot(TORQUE);
    while(isEmptyPointList(point_list))
    {
		printf("Move arm to each position and press any key to record position\n"
			   "or <space> to quit recording.\n");
		for(list_length=0;key!=' ';list_length++)
		{
			printf("\n\aPosition %d:",list_length);
			while(!kbhit())
			{
				setJointTorque(Robot,jtAry); 			
				updateRobot(Robot);
				compensateGravity(Robot);
				actuateRobot(Robot,(float)1.0, N_AXES_GCOMP);
			}			
			key=getch();
			if(key!=' ')
			{
				for(axis=0;axis<N_AXES;axis++)
				{
					get_position(axis,&n_axis_point[axis]);
					printf("%.1lf,",n_axis_point[axis]);
				}
				point_list=addPointListElement(point_list,n_axis_point);
			}
		}
		
		/* enables robot so that it will hold position */
		enableRobot(POSITION);

		/* decrements list_length to test for empty point list dr*/
        list_length--;
		key = '1';
		
		printf("\nlistlength= %d", list_length);
	} 
	
	return(point_list);
} 
     
void alignRobot(robottype robot){
    int axis;
    
    for(axis=0;axis<N_AXES;axis++){
    	printf("Searching for marker pulse on axis %d.\n",axis);
    	manMarkerFound(axis);
    	}  
    }



   		
/*==========================================================*
 * TORQUE CONTROL FUNCTIONS:                                *
 *==========================================================*/   
	
void actuateRobot(robottype robot, float outputLevel, int numberOfAxes){
    int axis, absolute_pos=0;
    float jt1, jt2;

     if(outputLevel<0 || outputLevel>1.0)
          _error("Out of range argument outputLevel in function actuateRobot().");

     for(axis=0;axis<numberOfAxes;axis++){

          /*------------------------------------------------------*
           * Get the motor torques by scaling the joint torques   *
           * according to the following Joint-Motor space torque  *
           * transformations:                              	      *                    						         
           *  | mt0 |     |-1/N1   0      0        0 | | jt0 |    *
           *  | mt1 |  =  | 0   1/2N2  -n3/2N2     0 | | jt1 |    *
           *  | mt2 |     | 0  -1/2N2  -n3/2N2     0 | | jt2 |    *
           *  | mt3 |     | 0      0      0      1/N4| | jt3 |    *
           *                                                      *
           *  | mt4 |  =  | 1/2N5    -n6/2N5     0 | | jt4 |	  *
           *  | mt5 |     | 1/2N5     n6/2N5     0 | | jt5 |      *
           *  | mt6 |     |    0         0    -1/N7| | jt6 |      *
           *------------------------------------------------------*/

          switch(axis){
              case 0:
               robot->mt = -((robot->jt)/N1);

          /*------------------------------------------------------*
           * If torque ripple compensation is specified, then add *
           * the appropriate torque from the torque ripple array. *
           * That torque is equal to the torque associated with   *
           * the present position minus the marker offset:        *
           *                                                      *
           *                  T = TR[mpos - mo]                   *
           *------------------------------------------------------*/
               if(COMPENSATE_TR)
				{
               		absolute_pos = robot->mpos - robot->mo;
               		if (absolute_pos<0) absolute_pos+=4096;
               		robot->mt += robot->tr[absolute_pos];
				}
          /*------------------------------------------------------*
           * Command the calculated motor torque scaled by the    *
           * scale factor, outputLevel.                           *
           *------------------------------------------------------*/
               setMotorTorque(axis,(int)(outputLevel*robot->mt));
               robot->jt=0;
               break;
               
              case 1:
               jt1 = robot->jt;
               break;
               
              case 2:
               jt2 = robot->jt;
               robot--;

               robot->mt = (jt1)/(2*N2) - DIF3*(jt2)/(2*N2);
               if(COMPENSATE_TR){
               	absolute_pos=robot->mpos-robot->mo;
               	if (absolute_pos<0) absolute_pos+=4096;
               	robot->mt +=robot->tr[absolute_pos];
				}
               setMotorTorque((axis-1),(int)(outputLevel*robot->mt));
               robot->jt=0;
               
               robot++;

               robot->mt = -(jt1)/(2*N2) - DIF3*(jt2)/(2*N2);
               if(COMPENSATE_TR){
               	absolute_pos=robot->mpos-robot->mo;
               	if (absolute_pos<0) absolute_pos+=4096;
               	robot->mt +=robot->tr[absolute_pos];
				}
               setMotorTorque(axis,(int)(outputLevel*robot->mt));
               robot->jt=0;
               break;
               
              case 3:
               robot->mt = ((robot->jt)/N4);
               if(COMPENSATE_TR){
               	absolute_pos=robot->mpos-robot->mo;
               	if (absolute_pos<0) absolute_pos+=4096;
               	robot->mt +=robot->tr[absolute_pos];
				}
               setMotorTorque(axis,(int)(outputLevel*robot->mt));
               robot->jt=0;
               break;
               
              case 4:
               jt1 = robot->jt;
               break;
               
              case 5:
               jt2 = robot->jt;
               robot--;

               robot->mt = (1/2*N5)*jt1 + (-DIF6/2*N5)*jt2;
               if(COMPENSATE_TR){
               	absolute_pos=robot->mpos-robot->mo;
               	if (absolute_pos<0) absolute_pos+=4096;
               	robot->mt +=robot->tr[absolute_pos];
				}
               setMotorTorque((axis-1),(int)(outputLevel*robot->mt));
               robot->jt=0;
               
               robot++;

               robot->mt = (1/2*N5)*jt2 + (DIF6/2*N5)*jt1;
               if(COMPENSATE_TR){
               	absolute_pos=robot->mpos-robot->mo;
               	if (absolute_pos<0) absolute_pos+=4096;
               	robot->mt +=robot->tr[absolute_pos];
				}
               setMotorTorque(axis,(int)(outputLevel*robot->mt));
               robot->jt=0;
               break;
               
              case 6:
              	robot->mt = -((robot->jt)/N7);
               	
               	/* Do not compensate for torque ripple */
               	//if(COMPENSATE_TR)
               	//{
                //		absolute_pos=robot->mpos-robot->mo;
                //		if (absolute_pos<0) absolute_pos+=4096;
                //		robot->mt +=robot->tr[absolute_pos];
				//}
               	setMotorTorque(axis,(int)(outputLevel*robot->mt));
               	robot->jt=0;
               	break;
               	
              default:
               _error("Invalid axis call in actuateRobot().");
              }

          robot++;
          }
     }


void setJointTorque(robottype robot, float jointTorques[N_AXES]){
	int axis;
    
    for(axis=0;axis<N_AXES;axis++){
    	robot->jt = jointTorques[axis];
    	robot++;
    	}
	}

void addJointTorque(robottype robot, float jointTorques[N_AXES]){
	int axis;
    
    for(axis=0;axis<N_AXES;axis++){
    	robot->jt += jointTorques[axis];
    	robot++;
    	}               
	}

void compensateGravity(robottype robot){
	int axis;
	robottype tmp=robot;
	float gt[N_AXES]=N_AXIS_ZERO_ARY;  
	float s[N_AXES]=N_AXIS_ZERO_ARY;
	float c[N_AXES]=N_AXIS_ZERO_ARY;
	
	for(axis=0;axis<N_AXES;axis++)
	{
		s[axis]=tmp->s;
		c[axis]=tmp->c; 
		tmp++;	
	}	
	
	gt[0]=(float)0.0;
 	gt[1]=(-c[1]*c[2]*c[3] + s[1]*s[3])*A + (c[3]*s[1]+c[1]*c[2]*s[3])*B - c[1]*c[2]*C - s[1]*D;
	gt[2]=(c[3]*s[1]*s[2])*A - s[1]*s[2]*s[3]*B + s[1]*s[2]*C;
	gt[3]=(-c[1]*c[3]+c[2]*s[1]*s[3])*A + (c[1]*s[3]+c[2]*c[3]*s[1])*B;
	
	if(N_AXES==7){
		gt[4]=(float)0.0;
		gt[5]=(float)0.0;
		gt[6]=(float)0.0;
	} 
	
	addJointTorque(robot, gt);
	}

void calibrateZeroG(robottype robot)
{
    float mt[N_AXES]=N_AXIS_GCOMP_ZERO_ARY;
    float sum[N_AXES]=N_AXIS_GCOMP_ZERO_ARY;
    int axis, test, i; 
    /* Tjik = torque about joint i in experiment k*/
    double Tj41,Tj42,Tj43;
    double Tj21,Tj22,Tj23;
    double Tj31,Tj32,Tj33;
    double AA,BB,CC,DD;
    
    double* ja;
    double ja0[N_AXES] = {0,0,0,0,};
    double ja1[N_AXES] = {0,-HALFPI,0,0};  
    double ja2[N_AXES] = {0,-HALFPI,0,HALFPI};
    double ja3[N_AXES] = {0,-HALFPI,-HALFPI,HALFPI};
	
	if(N_AXES==7)
	{
		for(axis=4;axis<N_AXES;axis++)
		{
			ja0[axis]=0;
			ja1[axis]=0;
			ja2[axis]=0;			
			ja3[axis]=0;
		}
	}
    
  
    printf("Self-calibrating gravity compensation constants...\n");
    
    /*----------------------------------------------------------*
 	 * Enable closed-loop position control on the axis.  Most   *  
 	 * likely, it is already in this mode, so the call may be   *
 	 * redundant, but that's ok.                                *
 	 *----------------------------------------------------------*/
   
    setupMove(SLOW_VEL,SLOW_ACCEL);
	enableRobot(POSITION);

	ja=ja0;
	commandMoveJoint(ja);
	while (!all_done()){};
	printf("\nDone moving to the zero position, press any key to move through calibration tests");
	getch();

    for(test=1;test<4;test++)
    {
    /*----------------------------------------------------------*
 	 * Move to the appropriate calibration position where axis  *   
 	 * 0 is arbitrary, but for now, we'll set it to zero:       *
 	 *                                                          *
 	 *   	        Axis1	Axis2	Axis3                       *
	 *		exp 1:  -90	    0	    0                           *
	 *		exp 2:	-90	    0	    90                          *
	 *		exp 3:	-90	    -90	    90                          *
 	 *----------------------------------------------------------*/  
 	 switch(test)
 	 	{
 	 	    case 1: 
 	 	        ja = ja1;
 	 	    	break;
 	 	    case 2:
 	 	        ja = ja2;
 	 	    	break;
 	 	    case 3:
 	 	        ja = ja3;
 	 	    	break;
            default:
 	 	    	_error("Error 1...Unsupported test case in demojoints().\n");
 	 	    	break; 	 	
 	 	}
            
	commandMoveJoint(ja);
	while (!all_done()){};
	printf("\nDone moving to the calibration position, press any key to determine parameters");
	getch();
	printf("\nCollecting Torque values");
	
	/*----------------------------------------------------------*
 	 * Grab the average motor torques at this position.         *
 	 *----------------------------------------------------------*/
	for(axis=1;axis<N_AXES_GCOMP;axis++) sum[axis]=0;
	
	for(i=0;i<TORQUE_SAMPLES;i++)
	{
		for(axis=1;axis<N_AXES_GCOMP;axis++)
		{
			mt[axis]=(float)getMotorTorque(axis); 
	    	sum[axis]+=(float)mt[axis];
	    }
		for(axis=1;axis<N_AXES_GCOMP;axis++)	printf("sum%d=%.1lf\t",axis,sum[axis]);
		printf("\n");
	}
    	
	for(axis=1;axis<N_AXES_GCOMP;axis++) mt[axis]=(float)(sum[axis]/TORQUE_SAMPLES);    

	/*----------------------------------------------------------*
 	 * Convert those motor torques to joint torques.            *
 	 *----------------------------------------------------------*/
 	 	switch(test){
 	 	    case 0:
 	 	    	break;
 	 	    case 1: 
 	 	     	Tj21 = (double)(N2*mt[1] - N2*mt[2]);
 	 	    	Tj31 = (double)(-(N2/DIF3)*mt[1] -(N2/DIF3)*mt[2]); 
 	 	    	Tj41 = (double)(N4*mt[3]);
 	 	    	break;      
 	 	    case 2:
 	 	    	Tj22 = (double)(N2*mt[1] -N2*mt[2]);
 	 	    	Tj32 = (double)(-(N2/DIF3)*mt[1] -(N2/DIF3)*mt[2]); 
 	 	    	Tj42 = (double)(N4*mt[3]);
 	 	    	break;
 	 	    case 3:
 	 	    	Tj23 = (double)(N2*mt[1] - N2*mt[2]);
 	 	    	Tj33 = (double)(-(N2/DIF3)*mt[1] - (N2/DIF3)*mt[2]);
 	 	    	Tj43 = (double)(N4*mt[3]);
 	 	    	break;
 	 	    default:
 	 	    	_error("Error 2...Unsupported test case in demojoints().\n");
 	 	    	break; 	 	
 	 		}  
 	 		
 	 } 
 
 	
 	/* prints out all torque values for comparison  */
 	
 	printf("\nExperiment #1: Tj21 = %.1f\tTj31 = %.1f\tTj41 = %.1f",Tj21, Tj31, Tj41);
 	printf("\nExperiment #2: Tj22 = %.1f\tTj32 = %.1f\tTj42 = %.1f",Tj22, Tj32, Tj42); 	
 	printf("\nExperiment #3: Tj23 = %.1f\tTj33 = %.1f\tTj43 = %.1f",Tj23, Tj33, Tj43); 	
 	getch();
 	/*----------------------------------------------------------*
 	 * Calculate the gravity compensation calibration constants,*
 	 * ABCD, from those measured joint torques: the first number*
 	 * is the number of the joint and the second number is the  *
 	 * number of the experiment.                                *
 	 *----------------------------------------------------------*/
    
	BB= -Tj41;
	DD= -Tj41   +   Tj21;
	AA= -Tj42;
	CC=  Tj33   -   Tj41;
 	 
 	/*----------------------------------------------------------*
 	 * Save ABCD to file or Print to screen                     *
 	 *----------------------------------------------------------*/
    printf("\nA=\t%.1f", AA);
	printf("\nB=\t%.1f", BB);
	printf("\nC=\t%.1f", CC);
	printf("\nD=\t%.1f", DD);
    
 	
 	printf("\nSelf-calibration completed.\n");
	getch();
}



/*==========================================================*
 * LIBRARY FUNCTIONS:  	                                    *
 *==========================================================*/
void _makeEmptyTorqueRippleAry(int** ary){
    int i;
    (*ary) = (int*)calloc(TR_ARY_LENGTH,sizeof(int));
    for (i=0; i<TR_ARY_LENGTH; i++) (*ary)[i]=0; 
	}


/*  history of function  */
/*	980708 - dr changed filename from 12 to 100 characters */   

void _makeTorqueRippleAry(int** ary, FILE* fp, char tr_data_filename[100]){
	size_t size;
	float av=0;
	    
    if ((fp=fopen(tr_data_filename, "rb"))==NULL ){
		printf( "Error opening file %s in this directory.\n", tr_data_filename);
		_error("");
		}

    (*ary) = (int*) calloc(TR_ARY_LENGTH, sizeof(int));
	if((*ary)!=NULL) size = fread((*ary), sizeof(int), TR_ARY_LENGTH, fp);
	else{
		printf("Insufficient memory available for %s\n",tr_data_filename);
		_error("");
		}
	if(size<TR_ARY_LENGTH){
	    printf("File %s contains only %d values\n",tr_data_filename, size);
		_error("");
		}
	/* Now that the file is open, close it.  It won't be needed again. */
	fclose(fp);                                  
	
                                       
/* use this to print filename to screen	printf("File %s : \n", tr_data_filename);*/
	}   
	
int _getSimulatedPosition(int axis){
    static int pos0=500,pos1=800,pos2=0,pos3=0;
    static int pos4=0,pos5=0,pos6=0;
    
    switch(axis){
     	case 0: 
     		pos0 = (pos0 < 4096) ? pos0+1 : 0;
     		return(pos0);
     		break;
     	case 1:
     		pos1 = (pos1 < 4096) ? pos1+2 : 0;
     		return(pos1);
     		break;
     	case 2:
     		pos2 = (pos2 < 4096) ? pos2+3 : 0;
     		return(pos2);
     		break;
     	case 3: 
     		pos3 = (pos3 < 4096) ? pos3+4 : 0;
     		return(pos3);
     		break;
     	case 4: 
     		pos4 = (pos4 < 4096) ? pos4+5 : 0;
     		return(pos4);
     		break;
     	case 5: 
     		pos5 = (pos5 < 4096) ? pos5+6 : 0;
     		return(pos5);
     		break;
     	case 6: 
     		pos6 = (pos6 < 4096) ? pos6+7 : 0;
     		return(pos6);
     		break;
     	default:
     		return(0);
     		break;    
    }    
  } 

/*==========================================================*
 * AWAKEN/SLEEP FUNCTIONS:  	                            *
 *==========================================================*/   
void Awaken1(void)
{
	int axis;
    
	printf("\nMove all joints to the zero position and press any key.");
	getch();
	for(axis=0; axis<N_AXES; axis++) set_position(axis,0);
	printf("\n\nMotors zeroed.");

	printf("\n\nMove all joints to the rest position and press any key.");
	getch();
	for (axis=0;axis<N_AXES; axis++) controller_run(axis);
}

char StillMoving2(void)
{
	return(1);
}	     
	

/************************************************************************
* awakenRobot() automatically finds its zero positions based on joint   *
* limits																*
* 980730 - dr transfered to library 									*
************************************************************************/

void awakenRobot(robottype robot)
{
    double mpos1Max, mpos1Min;
    double mpos2Max, mpos2Min;
    double n_axis_point[N_AXES]=N_AXIS_AWAKEN_ARY;
	double Joint5Adjustment[N_AXES]=N_AXIS_AWAKEN_ARY;
	double Joint5AdjustmentValue = -10500;
	int axis;
	
	/*Moves joint 5 to - pi/2 so that joint 6 can find stops */
	
	if(N_AXES==7)
	{
		Joint5Adjustment[4]=Joint5AdjustmentValue;
		Joint5Adjustment[5]=Joint5AdjustmentValue;
	}
		
    /*----------------------------------------------------------*
 	 * Set the present robot configuration to be the temporary  *
 	 * zero position.                                           *
 	 *----------------------------------------------------------*/
	for(axis=0;axis<N_AXES;axis++) set_position(axis,0); 
    
    /*----------------------------------------------------------*
 	 * move to semi-upright robot configuration with elbow      *
 	 * moved away from wrist to avoid the wrist contacting		*
 	 * the inner link											*
 	 *----------------------------------------------------------*/ 
	
	setupMove(SLOW_VEL,SLOW_ACCEL);
	enableRobot(POSITION);
    
    //dr added to test only wrist
    if(1)
    {
    commandMove(n_axis_point); 
    while(!all_done()){}  /* wait */     
	}
    
    if(N_AXES==7)
    {
    axis=6;
    printf("Locating zero position for joint axis %d\n",(axis+1));
          
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis,SLOW_VEL,SLOW_ACCEL);    /* start const vel */
	while(!isJointLimitReached(axis)){}  /* wait till joint limit reached*/
				
     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos1Max);
	printf("Joint %d maximum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/    
      
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);   /* start const vel*/
	while(isJointLimitReached(axis)){}	/*waits till motion in opposite direction is within limits*/
	while(!isJointLimitReached(axis)){}  /* wait till joint limit reached */
 
     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos1Min);
	printf("Joint %d minimum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/

	n_axis_point[axis] = (mpos1Max+mpos1Min)/2;

     /*----------------------------------------------------------*
      * Move to that position slowly.                            *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
    set_position(axis,0);
    n_axis_point[axis]=0;
	printf("Zeroed joint axis %d\n",(axis+1));

	axis=4;
    printf("Locating zero position for joint axis %d\n",(axis+1));

     /*==========================================================*
      * Determine the joint 5 zero position.  joint 5 is done    *
      * before joint 6 because the wrist cover takes away some   *
      * of the joint 6 space. by doing joint 5 first we are able *
      * to move to a position where joint 6 will not be affected *
      * by the wrist cover										 *
      *==========================================================*/
     
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	
    //dr removed due to joint location change in rev 99q1 wrist
//	v_move(axis,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
//	v_move(axis+1,-SLOW_VEL,SLOW_ACCEL);        /* start constvel */
//	while(!isJointLimitReached(axis) || !isJointLimitReached(axis+1)){}

     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/

//	get_position(axis, &mpos2Max);
//	get_position(axis+1, &mpos1Max);
//	printf("Joint %d maximum position found\n", (axis+1));
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/

//dr this was actually the positive direciton, changed velocity/accelerations to negative
	v_move(axis,-SLOW_VEL,-SLOW_ACCEL);     /* start const vel*/
	v_move(axis+1,-SLOW_VEL,-SLOW_ACCEL);       /* start const vel*/
	while(isJointLimitReached(axis) || isJointLimitReached(axis+1)){} /* waits till back within error limit*/
	while(!isJointLimitReached(axis) || !isJointLimitReached(axis+1)){} /* finds joint limit*/

     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos2Min);
	get_position(axis+1, &mpos1Min);
	printf("Joint %d minimum position found\n", (axis+1));
     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
 	  * the percentages were determined experimentally 			 *
	  * to have joint 5 in the middle of its workspace 			 *
	  *----------------------------------------------------------*/
	//dr changed the joint 5 stationary stop so the zero position needs to be modified			
//	n_axis_point[axis]=   mpos1Max*(0.465) + mpos1Min*(0.535); 
//	n_axis_point[axis+1]= mpos2Max*(0.465) + mpos2Min*(0.535); 

	//dr the value 10500 was determined experimentality to zero joint 5
	n_axis_point[axis]= mpos2Min + 10500;
	n_axis_point[axis+1]= mpos1Min + 10500;
	
	
	 /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
	printf("Zeroed joint axis %d\n", (axis+1));
                
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
	set_position(axis,0);
	set_position(axis+1,0);


     /*==========================================================*
      *Determine joint 6 zero position                           *
      *==========================================================*/
    axis=5;
    printf("Locating zero position for joint axis %d\n",(axis+1));  

     /*----------------------------------------------------------*
      * Once the posiiton is zeroed.  Move joint 6 to -pi/2 so   *
      * that the pitch is not affected by the clasp on the wrist *
      * cover.	The value was determined experimentally so that  *
      * when the pitch looks for its stops they will be an equal *
      * distance apart.											 *
      *----------------------------------------------------------*/

    commandMove(Joint5Adjustment);  
	while (!all_done()){};
     
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis-1,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	v_move(axis,SLOW_VEL,SLOW_ACCEL);        /* start constvel */
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/

	get_position(axis, &mpos2Min);
	get_position(axis-1, &mpos1Min);
	printf("Joint %d maximum position found\n",(axis+1));

     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis-1,SLOW_VEL,SLOW_ACCEL);     /* start const vel*/
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	while(isJointLimitReached(axis-1) || isJointLimitReached(axis)){}
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/

	get_position(axis, &mpos2Max);
	get_position(axis-1, &mpos1Max);
	printf("Joint %d minimum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/

	n_axis_point[axis-1]=(mpos1Max+mpos1Min)/2 - Joint5AdjustmentValue;
	n_axis_point[axis]  =(mpos2Max+mpos2Min)/2 - Joint5AdjustmentValue;	

     /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
	printf("Zeroed joint axis %d\n", (axis+1));

     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
	set_position(axis-1,0);
	set_position(axis,0);
	n_axis_point[axis]=0;
	n_axis_point[axis-1]=0;
	
	}

 	//dr added to test only wrist
 	if(1)
 	{
 	axis=3;
	printf("Locating zero position for joint axis %d\n",(axis+1));       
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);   /* start const vel*/
	while(!isJointLimitReached(axis)){}  /* wait till joint limit reached */
 
     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos1Min);
	printf("Joint %d minimum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
	 
	 
	 /* this point was experimentally determined by finding the *
      *  negative joint and then determining the zero position   */
      
    n_axis_point[axis] = mpos1Min + 11314;
     /*----------------------------------------------------------*
      * Move to that position slowly.                            *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
    set_position(axis,0);
    n_axis_point[axis]=0;
	printf("Zeroed joint axis %d\n",(axis+1));

	axis=2;
    printf("Locating zero position for joint axis %d\n",(axis+1));  

     /*==========================================================*
      *Determine joint 3 zero position                           *
      *==========================================================*/
     
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis-1,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);        /* start constvel */
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/

	get_position(axis, &mpos2Max);
	get_position(axis-1, &mpos1Max);
	
	
	printf("Joint %d maximum position found\n", (axis+1));
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/

	// hk added to remove drooping
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */


	v_move(axis-1,SLOW_VEL,SLOW_ACCEL);     /* start const vel*/
	printf("Moving in pos dirs axis 1\n");
	v_move(axis,SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	printf("Moving in pos dir axis 2\n");
	while(isJointLimitReached(axis-1) || isJointLimitReached(axis)){} /* waits till back within error limit*/
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){} /* finds joint limit*/

     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos2Min);
	get_position(axis-1, &mpos1Min);
	printf("Joint %d minimum position found\n", (axis+1));
     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
	n_axis_point[axis-1]=(mpos1Max+mpos1Min)/2;
	n_axis_point[axis]=(mpos2Max+mpos2Min)/2;
     /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
	printf("Zeroed joint axis %d\n", (axis+1));
                
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
	set_position(axis-1,0);
	set_position(axis,0);

     /*==========================================================*
      *Determine joint 2 zero position                           *
      *==========================================================*/

   printf("Locating zero position for joint axis %d\n",(axis));
     
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis-1,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	v_move(axis,SLOW_VEL,SLOW_ACCEL);        /* start constvel */
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

      /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos2Min);
	get_position(axis-1, &mpos1Min);
	printf("Joint %d maximum position found\n",axis);
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis-1,SLOW_VEL,SLOW_ACCEL);     /* start const vel*/
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
	while(isJointLimitReached(axis-1) || isJointLimitReached(axis)){}
	while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos2Max);
	get_position(axis-1, &mpos1Max);
	printf("Joint %d minimum position found\n", axis);
     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
	n_axis_point[axis-1]=(mpos1Max+mpos1Min)/2;
	n_axis_point[axis]=(mpos2Max+mpos2Min)/2;
     /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
	printf("Zeroed joint axis %d\n", axis);
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
	set_position(axis-1,0);
	set_position(axis,0);
	n_axis_point[axis]=0;
	n_axis_point[axis-1]=0;


	axis=0;
	printf("Locating zero position for joint axis %d\n",(axis+1));    
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis,SLOW_VEL,SLOW_ACCEL);    /* start const vel */
	while(!isJointLimitReached(axis)){}  /* wait till joint limit reached*/
				
     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos1Max);
   	printf("Joint %d maximum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
	v_move(axis,-SLOW_VEL,SLOW_ACCEL);   /* start const vel*/
	while(isJointLimitReached(axis)){}	/*waits till motion in opposite direction is within limits*/
	while(!isJointLimitReached(axis)){}  /* wait till joint limit reached */
 
     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
	get_position(axis, &mpos1Min);
	printf("Joint %d minimum position found\n", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
    n_axis_point[axis] = (mpos1Max+mpos1Min)/2;
    
     /*----------------------------------------------------------*
      * Move to that position slowly.                            *
      *----------------------------------------------------------*/
	commandMove(n_axis_point);
	while(!all_done()){}  /* wait */
    
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
    set_position(axis,0);
    n_axis_point[axis]=0;
	printf("Zeroed joint axis %d\n",(axis+1));
    }

}

void zeroAxisRepeatability(robottype robot)
{
    double mpos1Max, mpos1Min;
    double mpos2Max, mpos2Min;
    double n_axis_point[N_AXES]=N_AXIS_AWAKEN_ARY;
	int axis, i;
	double vel_reducer=4;
	double repeat_point[N_AXES][ZEROAXIS_SAMPLES];
	double sum;
		
    /*----------------------------------------------------------*
 	 * Set the present robot configuration to be the temporary  *
 	 * zero position.                                           *
 	 *----------------------------------------------------------*/
	for(axis=0;axis<N_AXES;axis++) set_position(axis,0); 
    
    /*----------------------------------------------------------*
 	 * move to semi-upright robot configuration.                     *
 	 *----------------------------------------------------------*/ 
	setupMove(SLOW_VEL,SLOW_ACCEL);
	enableRobot(POSITION);
    printf("\nEnable Robot");
    getch();
    
    commandMove(n_axis_point); 
    while(!all_done()){}  /* wait */     


    for(axis=N_AXES-1;axis>=0; /*axis is subtracted within loop based on different axes*/)
    {
    printf("\nLocating zero position for joint axis %d",(axis+1));

	switch(axis)
    {

	 	 /*==========================================================*
      * CASE:  Single axes....                                   *
      *==========================================================*/
    	case 3:

		for(i=0;i<ZEROAXIS_SAMPLES;i++)
		{
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/

			v_move(axis,-SLOW_VEL,SLOW_ACCEL);   /* start const vel*/
			while(!isJointLimitReached(axis)){}  /* wait till joint limit reached */
 
     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos1Min);
			printf("\nJoint %d minimum position found", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
	 
	 
	 /* this point was experimentally determined by finding the *
      *  negative joint and then determining the zero position   */
      
       		n_axis_point[axis] = mpos1Min + 11314;
           	repeat_point[axis][i]=n_axis_point[axis];

     /*----------------------------------------------------------*
      * Move to that position slowly.                            *
      *----------------------------------------------------------*/
			commandMove(n_axis_point);
			while(!all_done()){}  /* wait */
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
      }

            set_position(axis,0);
            n_axis_point[axis]=0;
			printf("\nZeroed joint axis %d",(axis+1));
            axis--;
               
            break;
	 
	 /*==========================================================*
      * CASE:  Single axes....                                   *
      *==========================================================*/
    	case 0:case 6:
          
		for(i=0;i<ZEROAXIS_SAMPLES;i++)
		{
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis,SLOW_VEL,SLOW_ACCEL);    /* start const vel */
			while(!isJointLimitReached(axis)){}  /* wait till joint limit reached*/
				
     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos1Max);
   			printf("\nJoint %d maximum position found", (axis+1));

     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis,-SLOW_VEL,SLOW_ACCEL);   /* start const vel*/
			while(isJointLimitReached(axis)){}	/*waits till motion in opposite direction is within limits*/
			while(!isJointLimitReached(axis)){}  /* wait till joint limit reached */
 
     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos1Min);
			printf("\nJoint %d minimum position found", (axis+1));

     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
            switch(axis)
            {
            	case 3:   /* these values were determined experimentally  */
            		n_axis_point[axis] = (0.773)*mpos1Min + (0.227)*mpos1Max;
                	repeat_point[axis][i]=n_axis_point[axis];
                	break;
            	default:    /* symmetric joints */
            		n_axis_point[axis] = (mpos1Max+mpos1Min)/2;
                    repeat_point[axis][i]=n_axis_point[axis];
                break;
            }
     /*----------------------------------------------------------*
      * Move to that position slowly.                            *
      *----------------------------------------------------------*/
			commandMove(n_axis_point);
			while(!all_done()){}  /* wait */
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
      }
      
            set_position(axis,0);
            n_axis_point[axis]=0;
			printf("\nZeroed joint axis %d",(axis+1));
            axis--;
               
            break;
          
     /*==========================================================*
      * CASE:  DIFFERENTIAL AXIS...                              *
      *==========================================================*/
		case 2:case 5:
     /*==========================================================*
      *Determine the joint 2 and joint 5                         *
      *==========================================================*/
     
      for(i=0;i<ZEROAXIS_SAMPLES;i++)
      {
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis-1,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
			v_move(axis,-SLOW_VEL,SLOW_ACCEL);        /* start constvel */
			while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

      /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/

			get_position(axis, &mpos2Max);
			get_position(axis-1, &mpos1Max);
    		printf("\nJoint %d maximum position found", (axis+1));
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis-1,SLOW_VEL,SLOW_ACCEL);     /* start const vel*/
			v_move(axis,SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
			while(isJointLimitReached(axis-1) || isJointLimitReached(axis)){} /* waits till back within error limit*/
			while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){} /* finds joint limit*/

     /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos2Min);
			get_position(axis-1, &mpos1Min);
			printf("\nJoint %d minimum position found", (axis+1));
     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
			n_axis_point[axis-1]=(mpos1Max+mpos1Min)/2;
			n_axis_point[axis]=(mpos2Max+mpos2Min)/2;
     /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
			commandMove(n_axis_point);
			while(!all_done()){}  /* wait */
			printf("\nZeroed joint axis %d.", (axis+1));
                

     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
//			set_position(axis-1,0);
//			set_position(axis,0);
 
     /*==========================================================*
      *Determine the joint 1 and joint 4                         *
      *==========================================================*/
     
     /*----------------------------------------------------------*
      * Move the axis in the negative direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis-1,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
			v_move(axis,SLOW_VEL,SLOW_ACCEL);        /* start constvel */
			while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

      /*----------------------------------------------------------*
      * Call this position the negative position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos2Min);
			get_position(axis-1, &mpos1Min);
			printf("\nJoint %d maximum position found",axis);
     /*----------------------------------------------------------*
      * Move the axis in the positive direction until the joint  *
      * limit is reached.                                        *
      *----------------------------------------------------------*/
			v_move(axis-1,SLOW_VEL,SLOW_ACCEL);     /* start const vel*/
			v_move(axis,-SLOW_VEL,SLOW_ACCEL);       /* start const vel*/
			while(isJointLimitReached(axis-1) || isJointLimitReached(axis)){}
			while(!isJointLimitReached(axis-1) || !isJointLimitReached(axis)){}

     /*----------------------------------------------------------*
      * Call this position the positive position limit.          *
      *----------------------------------------------------------*/
			get_position(axis, &mpos2Max);
			get_position(axis-1, &mpos1Max);
			printf("\nJoint %d minimum position found", axis);
     /*----------------------------------------------------------*
      * Define the relationship between the axis extremes and    *
      * the zero position.                                       *
      *----------------------------------------------------------*/
			n_axis_point[axis-1]=(mpos1Max+mpos1Min)/2;
			n_axis_point[axis]=(mpos2Max+mpos2Min)/2;
			repeat_point[axis-1][i]=n_axis_point[axis-1];
			repeat_point[axis][i]=n_axis_point[axis];
			
     /*----------------------------------------------------------*
      * Move to that position.                                   *
      *----------------------------------------------------------*/
			commandMove(n_axis_point);
			while(!all_done()){}  /* wait */
			printf("\nZeroed joint axis %d.", axis);
     /*----------------------------------------------------------*
      * Once there, zero the position.                           *
      * set_position sets the command and actual position to the *
      * desired position 										 *
      *----------------------------------------------------------*/
		}
			set_position(axis-1,0);
			set_position(axis,0);
			n_axis_point[axis]=0;
			n_axis_point[axis-1]=0;
			axis-=2;
			break;
          
          default:
               break;
		}
	}


	for(axis=0;axis<N_AXES;axis++)
	{
		printf("\nAxis %d\n", axis);
//		for(i=0;i<ZEROAXIS_SAMPLES;i++)	printf("%d:%.1f\t", i, repeat_point[axis][i]);
//		printf("\n");
		sum=0;
		for(i=0;i<ZEROAXIS_SAMPLES;i++)	sum=sum+repeat_point[axis][i];
		sum=sum/ZEROAXIS_SAMPLES;
		for(i=0;i<ZEROAXIS_SAMPLES;i++)	printf("%d:%.1f\t", i, (sum-repeat_point[axis][i]));
		printf("\n");
	}





}



char isJointLimitReached(int axis){ 
	double posError=0;
    /*----------------------------------------------------------*
 	 * Since robot could be applying a torque against the       *
 	 * ground, this check should be kinematically based.        *
 	 *----------------------------------------------------------*/
	get_error(axis, &posError);
	return((fabs(posError)>JOINT_LIMIT_POSITION_ERROR) ? TRUE : FALSE);
	}


/************************************************************************
*  	This program can only be used after a program that calibrates		*
*	the zero position.  This program moves the robot to a sleep 		*
*	positin that is relative to the zero position of the arm being		*
*	verticaly aligned.													*
*																		*
*	980708 - dr defines positions for both 4 and 7 axes now				*
************************************************************************/

void Sleep1(void){           
	clock_t t_init;
    
    /* experimentally determined and can be changed if desired */
    double ja_sleep[N_AXES]=SLEEP_ANGLE_ARY;
    
	/* get current time */
	t_init=clock()/1000;

	setupMove(SLEEP_VEL, SLEEP_ACCEL);
	enableRobot(POSITION);

	commandMoveJoint(ja_sleep);
	printf("\nSleeping...\n");

	while(!all_done()){
//       	if(kbhit()){                        /* for safety, kill axes */
//      		_error("<<Execution terminated.  Performing emergency kill...>>\n");
//	   		}
		if(clock()/1000>t_init+SLEEP_TIMEOUT){    /* if timeout */
       		_error("<<Sleep1() timed out.  Performing emergency kill...>>\n");
            }
		}
	}
/*---------------------------------------------------------------*
 * Note that the effective damping coefficient is highly         *
 * dependent on the cycle time:  more axes applied increases the *
 * cycle time.                                                   *
 *---------------------------------------------------------------*/
void Sleep2(robottype Robot)
{
	int axis, SleepTime=20; 
	extern int list_length;
    float jtAry[N_AXES]=N_AXIS_ZERO_ARY;  
	clock_t t_init;
	char	key;

	enableRobot(TORQUE);
	if(N_AXES==7)
	{
	 	controller_run(4);
	 	controller_run(5);
	 	controller_run(6);
	}

	printf("\nPress <q> when robot is in a rest position.\n");
	t_init=clock()/CLOCKS_PER_SEC;            
	while(key!='q'/*&& ((float)(clock())/CLOCKS_PER_SEC)<(t_init+SleepTime)*/)
	{
		if(kbhit())	key=getch();
		setJointTorque(Robot,jtAry); 
		updateRobot(Robot);
		compensateGravity(Robot);
		actuateRobot(Robot,(float)(SLEEP_SCALE_FACTOR), N_AXES);
    }
	
	for(axis=0;axis<N_AXES;axis++) controller_idle(axis);		

}

void Sleep4(void)
{
	  killRobot();
}

/*==========================================================*
 * POINTLIST FUNCTIONS                                      *
 *==========================================================*/
listtype makePointList(void)
{
	return(NULL);
}
		
listtype makePointListElement(double* Data)
{
	listtype element;
	int axis;
	
	element=(listtype)malloc(sizeof(struct Element));
    if(element==NULL) _error("Unable to allocate Element in makePointListElement().");
    
    /*--------------------------------------------------------*
     * Copy the contents of Data into the element.            *
     *--------------------------------------------------------*/
    for(axis=0;axis<N_AXES;axis++)  element->data[axis]=Data[axis];  
    
    /*--------------------------------------------------------*
     * Initialize the pointer to the next element.            *
     *--------------------------------------------------------*/
    element->next=NULL;
    return(element);
	} 

char isEmptyPointList(listtype list)
{
    return((list==NULL) ? TRUE : FALSE);
}     
	
void destroyPointList(listtype list)
{
   	if(!isEmptyPointList(list))
   	{
   		destroyPointList(list->next);
   	    free(list);
   	}
}

listtype addPointListElement(listtype list, double* data)
{
    listtype tmp;
    	/*--------------------------------------------------------*
    	 * Check to see if we are already at the end of the list. *
    	 *--------------------------------------------------------*/    
    if(isEmptyPointList(list)) list = makePointListElement(data);
    else 
    {
       	/*--------------------------------------------------------*
    	 * Find the end of the list by marching tmp through the   *
    	 * links until it finds the empty list.                   *
    	 *--------------------------------------------------------*/
        for(tmp = list; !isEmptyPointList(tmp->next); tmp = tmp->next); 
        /*--------------------------------------------------------*
    	 * Tack an element on the end...                          *
    	 *--------------------------------------------------------*/
         tmp->next = makePointListElement(data);
    }
    return(list);
}

void printPointList(listtype originalList){
	int i,axis; 
	listtype list;
	
	/*----------------------------------------------------------------------*
 	 * Make a copy of the list:                                             *
 	 *----------------------------------------------------------------------*/
	list = originalList;
	
	if(isEmptyPointList(list)) printf("Error...empty list in printPointList().\n");
	else{
		for(i=0;!isEmptyPointList(list);i++){ 
    		printf("point[%d]=\t",i);
    		for(axis=0;axis<N_AXES;axis++){
    			printf("%.1f\t",list->data[axis]); 
    			}
    		printf("\n");
    		list=list->next;
    		}
    	}
	}

/* Note:  the zeroth element corresponds to that which holds the first piece of data */
double* extractPointListData(listtype list, int element){
	listtype tmp; 
	int i=0;

	/*------------------------------------------------------------*
     * Find the element by marching tmp through the list until    *
     * the element is reached or until it finds an empty list,    *
     * in which case, error out.                                  *
     *------------------------------------------------------------*/ 
    i=0;
	for(tmp = list; !isEmptyPointList(tmp->next) && i < element; tmp = tmp->next) i++;
	
	if(isEmptyPointList(tmp->next) && i < element) 
		_error("Request for nonexistent element in extractPointListData().");
		
 	return(tmp->data);
} 	
          



/*==========================================================*
 * FILE SAVE/LOAD FUNCTIONS                                 *
 *==========================================================*/
/*--------------------------------------------------------------*
 * saveTrajectoryFile											*
 * 	                											* 
 * Saves trajectories to text file with a .trj extension.   	*
 * A date header is added.  File format is:                     *
 * 																*
 * date header													*
 * number of axes												*
 * point list length											*
 * point list data, tab separated								*  
 * 																*
 * Arguments:	char* filename  -- filename string				*
 *																* 
 * Return value:  void                                          *
 *																*
 * Usage:	saveTrajectoryFile("data");							*
 *--------------------------------------------------------------*/		
int saveTrajectoryFile(char *filename)
{
	FILE *fp;
	int axis, err;
	int i;
	time_t* tp;
	time_t t;
	listtype list;
    
    /*----------------------------------------------------------------------*
 	 * If filename is empty, prompt for filename.                           *
 	 *----------------------------------------------------------------------*/	
//	fflush(stdin);
	if(strlen(filename)==0){
		printf("What filename (8 characters max)?\n");
	    scanf("%s",filename);
	    fflush( stdin);
	    }
	    
	/*----------------------------------------------------------------------*
 	 * Make a copy of the list:                                             *
 	 *----------------------------------------------------------------------*/	
	list = point_list; 
		
	strcat(filename,".trj");
	
	/*----------------------------------------------------------------------*
 	 * Open the file.                                                       *
 	 *----------------------------------------------------------------------*/	
	fflush(stdin);
	if ((fp=fopen(filename, "w"))==NULL ){
		printf( "Error opening file %s in this directory.\n", filename);
		return(1); 	/* Error */
		}
			
  	/*----------------------------------------------------------------------*
 	 * Write date header.                                                   *
 	 *----------------------------------------------------------------------*/
  	fprintf(fp,"File created on:\n");
    t=time(tp);
  	if(t==-1) fprintf(fp,"Time not available");
  	else fprintf(fp,asctime(localtime(&t)));
	//printf("Current file position=%d",ftell(fp));  
	
	/*----------------------------------------------------------------------*
 	 * Write data.                                                          *
 	 *----------------------------------------------------------------------*/
	fprintf(fp,"%d\n",N_AXES);
	fprintf(fp,"%d\n",list_length);
	for(i=0;i<list_length;i++){
		for(axis=0;axis<N_AXES;axis++) fprintf(fp,"%.1f\t",list->data[axis]);
		fprintf(fp,"\n");
		list=list->next;
		}
	
	/*----------------------------------------------------------------------*
 	 * Close the file.                                                      *
 	 *----------------------------------------------------------------------*/	
	
	err=fclose(fp);
	if (err == EOF) perror("close failed");
//	fflush(stdout);
//	fflush(stdin);
//	filename="";
//	printf("filename %s\n\n", filename);
			
	return(0);	/* Success */	
	}
    
    
/*--------------------------------------------------------------*
 * loadTrajectoryFile											*
 * 	                											* 
 * Loads trajectories from text files created with 				*
 * saveTrajectoryFile() and with a .trj extension.   			*
 * File format is:                     							*
 * 																*
 * date header													*
 * number of axes												*
 * point list length											*
 * point list data, tab separated								*  
 * 																*
 * Arguments:	char* filename  -- filename string				*
 *																* 
 * Return value:  void                                          *
 *																*
 * Usage:	loadTrajectoryFile("data");							*
 *--------------------------------------------------------------*/	
int loadTrajectoryFile(char* filename){
	FILE *fp;
	int axis;
	int i, n_axes, err;
	double n_axis_point[N_AXES]; 
    
    fflush(stdin);
    
	/*----------------------------------------------------------------------*
 	 * If filename is empty, prompt for filename.                           *
 	 *----------------------------------------------------------------------*/	
	if(strlen(filename)==0){
		printf("What filename (8 characters max)?\n");
	    scanf("%s",filename);
	    fflush( stdin);
	    } 

	/*----------------------------------------------------------------------*
 	 * Open the trajectory file (filename.trj).  If not found or error,     *
 	 * return error code.                                                   *
 	 *----------------------------------------------------------------------*/	
	strcat(filename,".trj");
	if ((fp=fopen(filename, "r"))==NULL ){
		printf( "Error opening file %s in this directory.\n", filename);
		return(1);  /* Error */
		} 
	
	printf("Loading trajectory from file %s...\n",filename); 
	
	/*----------------------------------------------------------------------*
 	 * Skip past date header (constant 44 file positions from SEEK_SET=0).  *
 	 *----------------------------------------------------------------------*/
	fseek(fp, 44, SEEK_SET);
	
	/*----------------------------------------------------------------------*
 	 * Check to see that number of axes on file matches number of axes in   *
 	 * system.                                                              *
 	 *----------------------------------------------------------------------*/		
	fscanf(fp,"%d\n",&n_axes); 
	if(n_axes!=N_AXES){
		system("cls");
		printf("The number of axes on file does not match the number of axes in use.\n");
		printf("Loading of %s will be aborted.\n\n\n",filename); 
		_error("");
		} 
	
	/*----------------------------------------------------------------------*
 	 * If so, read data.                                                    *
 	 *----------------------------------------------------------------------*/
	fscanf(fp,"%d\n",&list_length);
	for(i=0;i<list_length;i++){
		for(axis=0;axis<n_axes;axis++){
			fscanf(fp,"%lf\t",&(n_axis_point[axis]));
			}
		point_list=addPointListElement(point_list,n_axis_point);
		}
	
	/*----------------------------------------------------------------------*
 	 * Close the file.                                                      *
 	 *----------------------------------------------------------------------*/	
	err=fclose(fp);
	if (err == EOF) perror("close failed");
	
	printf("n_axes=%d\n",n_axes);
	printf("list_length=%d\n",list_length);
	printPointList(point_list);
	fflush( stdin);
	    
    return(0);  /* Success */
	} 



/*==========================================================*
 * OLD STYLE, ARRAY-BASED FUNCTIONS:                        *
 *==========================================================*/

/* Get motor spindle angle in radians, 0 < angle < 2*pi.           *
 * Angle returned is relative to the position the axis was zeroed. */
float getMotorAngle(int axis){ 
	return((float)(getAutoRangedPosition(axis)*((2*PI)/4096.0)));
	}

/*----------------------------------------------------------------------*
 * Motor-Joint space torque transformations:                            *
 *                                                                      *
 *  | jt0 |     |-N1   0     0      0 | | mt0 |                   		*
 *  | jt1 |  =  | 0   N2    -N2     0 | | mt1 |                         *
 *  | jt2 |     | 0  -N2/n3 -N2/n3  0 | | mt2 |                         *
 *  | jt3 |     | 0    0     0     -N4| | mt3 |                         *
 *                                                                      *
 *  | jt4 |  =  |  N5     N5     0 | | mt4 |  		                    *
 *  | jt5 |     | -N5/n6  N5/n6  0 | | mt5 |                            *
 *  | jt6 |     |   0     0     -N7| | mt6 |                            *
 *----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*
 * Joint-Motor space torque transformations:                            *
 *                                                                      *
 *  | mt0 |     |-1/N1   0      0        0 | | jt0 |					*
 *  | mt1 |  =  | 0   1/2N2  -n3/2N2     0 | | jt1 |                    *
 *  | mt2 |     | 0  -1/2N2  -n3/2N2     0 | | jt2 |                    *
 *  | mt3 |     | 0      0      0     -1/N4| | jt3 |                    *
 *                                                                      *
 *  | mt4 |  =  | 1/2N5  -n6/2N5    0 | | jt4 |                        *
 *  | mt5 |     | 1/2N5   n6/2N5    0 | | jt5 |                        *
 *  | mt6 |     |    0      0    -1/N7| | jt6 |                        * 
 *                                                                      *
 *----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*
 * Motor-Joint space position transformations:          				*
 *                                                      				*
 *  | ja0 |     |-1/N1   0       0        0 | | ma0 |					*
 *  | ja1 |  =  | 0   1/2N2    -1/2N2     0 | | ma1 | 					*
 *  | ja2 |     | 0  -n3/2N2  -n3/2N2     0 | | ma2 | 					*	
 *  | ja3 |     | 0      0       0     -1/N4| | ma3 |					*
 *                                                       				*
 *  | ja4 |  =  |   1/2N5    1/2N5     0 | | ma4 | 						*
 *  | ja5 |     | -n6/2N5   n6/2N5     0 | | ma5 | 						*
 *  | ja6 |     |    0        0     -1/N7| | ma6 |						*
 *																		* 
 *----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*
 * Joint-Motor space position transformations:                          *
 *                                                                      *
 *  | ma0 |     |-N1     0      0        0 | | ja0 |        			*
 *  | ma1 |  =  | 0      N2   -N2/n3     0 | | ja1 |                    *
 *  | ma2 |     | 0     -N2   -N2/n3     0 | | ja2 |                    *
 *  | ma3 |     | 0      0      0       -N4| | ja3 |                    *
 *                                                                      *
 *  | ma4 |  =  |  N5  -N5/n6     0 | | ja4 | 		                    *
 *  | ma5 |     |  N5   N5/n6     0 | | ja5 |        		            *
 *  | ma6 |     | 0       0      -N7| | ja6 |               		    	*
 *----------------------------------------------------------------------*/
 
void setTorques(float JointSpaceComponent[], float MotorSpaceComponent[], float ScaleFactor)
{
    int axis;
    float MotorTorqueCommand[N_AXES]=N_AXIS_ZERO_ARY;

	

    MotorTorqueCommand[0]= MotorSpaceComponent[0]+(-JointSpaceComponent[0]/N1);
 	MotorTorqueCommand[1]= MotorSpaceComponent[1]+( JointSpaceComponent[1]/(2*N2)) - (DIF3*JointSpaceComponent[2])/(2*N2);
	MotorTorqueCommand[2]= MotorSpaceComponent[2]+(-JointSpaceComponent[1]/(2*N2)) - (DIF3*JointSpaceComponent[2])/(2*N2);
	MotorTorqueCommand[3]= MotorSpaceComponent[3]+(-JointSpaceComponent[3]/N4);
    
    if(N_AXES==7)
	{
	 	MotorTorqueCommand[4]= MotorSpaceComponent[4]+((JointSpaceComponent[4])/(2*N5) - DIF6*JointSpaceComponent[5]/(2*N5));
		MotorTorqueCommand[5]= MotorSpaceComponent[5]+((JointSpaceComponent[4])/(2*N5) + DIF6*JointSpaceComponent[5]/(2*N5));
		MotorTorqueCommand[6]= MotorSpaceComponent[6]+( -JointSpaceComponent[6]/N7);
	}
    
    for(axis=0;axis<N_AXES;axis++)
    {
		setMotorTorque(axis,(int)(ScaleFactor*MotorTorqueCommand[axis]));  
	}
}

void getAngles(float JointAngle[], float MotorAngle[], int MotorPosition[]){
	int axis;


	for(axis=0;axis<N_AXES;axis++)
	{
		MotorPosition[axis]=getAutoRangedPosition(axis);  
		MotorAngle[axis]=MotorPosition[axis]*CTS2RAD;
	}


	JointAngle[0]=(-MotorAngle[0] /  N1);
 	JointAngle[1]=( MotorAngle[1] / (2 * N2) - MotorAngle[2] / ( 2 * N2));
	JointAngle[2]=(-(DIF3*MotorAngle[1]) / (2*N2) - (DIF3*MotorAngle[2]) / (2*N2));
	JointAngle[3]=(-MotorAngle[3] / N4);

	if(N_AXES>4)
	{
		JointAngle[4]=( MotorAngle[4] / (2 * N5) + MotorAngle[5] / ( 2 * N5));
		JointAngle[5]=-(DIF3*MotorAngle[4])/(2*N5) + (DIF3*MotorAngle[5])/(2*N5);
		JointAngle[6]=-MotorAngle[6]/N7;
	}


	}

/* Determines position wrt robot frame 0, not floor */
void getCartesianEndEffectorPosition(float x[3], float s[4], float c[4])
{
    float s1c0,s1s0,term1,term2;

    if(N_AXES>4)
    {
		printf("Too many axes specified in getCartesianEndEffectorPosition().  deinitRobotializing...\n");
		deinitRobot();
		exit(0);
	}

	s1c0=s[1]*c[0];
    s1s0=s[1]*s[0];
    term1=c[2]*c[1]*c[0]-s[2]*s[0];
    term2=c[2]*c[1]*s[0]+s[2]*c[0];

    x[X]=(float)(-L3*s1c0+D3*term1-D4*(c[3]*term1-s[3]*s1c0)-L4*(c[3]*s1c0+s[3]*term1));
    x[Y]=(float)(-L3*s1s0+D3*term2-D4*(c[3]*term2-s[3]*s1s0)-L4*(c[3]*s1s0+s[3]*term2));
    x[Z]=(float)(L3*c[1]+D3*c[2]*s[1]-D4*(c[3]*c[2]*s[1]+s[3]*c[1])+L4*(c[3]*c[1]-s[3]*c[2]*s[1]));

}

char StillMoving(float vel[])
{     /* Needs work */
	int axis;
	float sum=0;

	for(axis=0; axis<N_AXES; axis++) sum=sum+vel[axis];

	if(sum>MOTION_EPSILON) return(TRUE);
	else return(FALSE);
}


/**********************************************************************
 * MARKERFOUND():                                                     *
 * This function increments the motor position and looks for the      *
 * index+ line to go high and the index- line to go low.  A true 	  *
 * condition is returned when the index pulse is located.             *
 *                                            						  *
 *                                                                    *
 * Although MEI provides home functions that can access the marker    *
 * pulse directly from the wired inputs for homing, the high          *
 * resolution of the motors mean that a high marker pulse signal      *
 * can be easily missed by the MEI board using that function.         *
 * Instead, the signal is routed into the analog input channel        *
 * corresponding to the axis number.  The ADC acts as a crude sample  *
 * and hold, allowing the marker pulse to be found, probably to       *
 * within a few encoder counts.                                       *
 *                                                                    *
 * NOTE:  axis 0 is currently used for frames						  *
 **********************************************************************/
char MarkerFound(int axis){
    int io=0;
    int bit_high=0, bit_low=0, MarkerPulseError = 10;
    double pos;
    int done=FALSE;
    double marker=0, marker1=0, marker2=0;
//	int error;
//	clock_t t_init;
    
    controller_run(axis);

//   	error=init_io(1,IO_INPUT); // DR removed because when a port is set to input/output all other bits are cleared
//  	error=init_io(0,IO_INPUT);
    get_position(axis, &pos);
    
    
                                                          
	do 
	{   
		/* Find marker in first place */
		while ( !((bit_high=bit_on(axis)) && !(bit_low=bit_on(axis + 8))) ) 
		{
		 	pos++;
		 	start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);  
		 	while(!axis_done(axis))	
		 		if(kbhit()) _error("Error...Execution terminated by user.  Exiting...\n");
			printf("Axis %d positive direction position %.1lf: Index+ =%d   Index- = %d\r",axis, pos,bit_high, bit_low);
		}                                    

		get_position(axis, &marker1);
		printf("\nFound marker pulse at %f\n", marker1);
		
		/* Check marker */
		pos+=100;
		start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);                                       	
		while(!axis_done(axis))	
			if(kbhit()) _error("Error...Execution terminated by user.  Exiting...\n");
		while (   !((bit_high=bit_on(axis))  && !(bit_low=bit_on(axis+8))) && (marker1<pos+100)) 
		{
		 	pos--;
		 	start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);  
		 	while(!axis_done(axis))	
		 		if(kbhit()) _error("Error...Execution terminated by user.  Exiting...\n");
		printf("Axis %d negative direction position %.1lf: Index+ =%d   Index- = %d\r",axis, pos,bit_high, bit_low);

		}                                    
        get_position(axis, &marker2);
        if (fabs(marker1-marker2)< MarkerPulseError) 
        {
        	done=TRUE;                                  
        	marker = (int)(marker1+marker2)/2;
			printf("\nMarker found consistantly at %f\n", marker);
		}
		else
		{
			printf("\nFalse marker pulse!\n");
		}  
	}
	while (!done);
	set_position(axis,0);                                     
   	return(TRUE);
                                                          
/*    for(;!(bit=bit_on(axis)) && !kbhit();pos++){
		printf("Axis position %.1lf: bit %d=%d\r",pos,axis,bit);

		start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);
		while(!axis_done(axis)){
			if(kbhit()){
				_error("Error...Execution terminated by user.  Exiting...\n");
				}
			}
		}

    set_position(axis,0);
  	return(TRUE); */
}  
	              


/*****************************************************************
 * MarkerFoundOffset will find the marker pulse using the index+ *
 * and index- automaically.  It will return the offset value     *
 *****************************************************************/
 
int MarkerFoundOffset(int axis)
{
    int io=0;
    int bit_high=0, bit_low=1, MarkerPulseError = 10;
    double pos;
    int done=FALSE;
    double marker=0, marker1=0, marker2=0;
    
    controller_run(axis);

//   	init_io(0,IO_INPUT);  // DR removed because when a port is set to input/output all other bits are cleared
//   	init_io(1,IO_INPUT);
    get_position(axis, &pos);
    printf("\n");
                                                          
	do 
	{   
		/* Find marker in first place */
		while (!((bit_high=bit_on(axis)) && !(bit_low=bit_on(axis + 8)))) 
		{
		 	pos++;
		 	start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);  
		 	/* remove when done debugging */
		 	printf("\rPosition %.1f",pos);
		 	
		 	while(!axis_done(axis))	
		 		if(kbhit()) _error("\nError...Execution terminated by user.  Exiting...");
		}                                    
		get_position(axis, &marker1);
		printf("\nFound marker pulse at %f", marker1);
		
		/* Check marker */
		pos+=100;
		start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);                                       	
		while(!axis_done(axis))	
			if(kbhit()) _error("\nError...Execution terminated by user.  Exiting...");
		while (   !((bit_high=bit_on(axis))  && !(bit_low=bit_on(axis+8))) && (marker1<pos+100)) 
		{
		 	pos--;
		 	start_move(axis,pos,SLOW_VEL,SLOW_ACCEL);  
		 	while(!axis_done(axis))	
		 		if(kbhit()) _error("\nError...Execution terminated by user.  Exiting...");
		}                                    
        get_position(axis, &marker2);
        if (fabs(marker1-marker2)< MarkerPulseError) 
        {
        	done=TRUE;                                  
        	marker = (int)(marker1+marker2)/2;
			printf("\nMarker found consistantly at %f", marker);
		}
		else
		{
			printf("\n\nFalse marker pulse!");
			printf("\nFinding marker pulse again for axis %d.\n", axis);
		}  
	}
	while (!done);
	marker=fmod(marker,4096);
	if (marker<0.0) marker+=4096;                                     
   	return((int)marker);
}








void getGravityCompTorque(float jt[], float s[], float c[])
{
	jt[0]=(float)0.0;
 	jt[1]=(-c[1]*c[2]*c[3] + s[1]*s[3])*A + (c[3]*s[1]+c[1]*c[2]*s[3])*B - c[1]*c[2]*C - s[1]*D;
	jt[2]=(c[3]*s[1]*s[2])*A - s[1]*s[2]*s[3]*B + s[1]*s[2]*C;
	jt[3]=(-c[1]*c[3]+c[2]*s[1]*s[3])*A + (c[1]*s[3]+c[2]*c[3]*s[1])*B;
}                 
 	                                        
void getTrig(float s[], float c[], float ja[])
{
	int axis;
 	for(axis=0;axis<N_AXES;axis++){
		s[axis]=(float)sin( ja[axis] );
		c[axis]=(float)cos( ja[axis] );
		}
}    
   
void timeDelay(int delay)
{           
	clock_t t_init;
	/* get current time */
	t_init=clock()/1000;
	while (clock()/1000 < t_init + delay){ /*wait*/}
       		
}
 