/*********************************************************************************/
/*		Barrett Technology Inc.													 */
/*		StandardC.c 	v0.1													 */
/*		Created:  	11/09/95 YM													 */
/*		Edited:		12/06/95 YM Added multiple commands and comments			 */ 
/*				02/13/96 YM Added GoToDifferentPositions						 */
/*				02/15/96 YM Modified Get to respond to 'G'						 */
/*				02/15/96 YM Added command motor on/off for NGK					 */ 
/*				02/19/96 YM Added PID functions(hidden) for NGK					 */
/*				06/05/97 JJ Changed formatting while GTK						 */
/*				01/17/98 DR Changed Get() to return proper value				 */
/*********************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <dos.h>
#include "stdc.h"
#include "serial.h"

int StrainStream(char motorflag, int duration)
{
	int err = 0;
	time_t t;
	long stoptime;
	time(&t);
	stoptime = t + duration;
	sprintf(chbuf, "%sFGET CS", &motorflag);
	while (t < stoptime)
	{
		asiString(chbuf);
		flushSerial();
		err = err_prompt_detection();
		if (err != 0) return err;
		time(&t);
	}
}

int Open(char motorflag) 					
{
	sprintf(chbuf, "%sO", &motorflag);
	asiString(chbuf);	
	return err_prompt_detection(); 	    		/* Check for return error value */	
}	
        

int Close(char motorflag)    				
{
	sprintf(chbuf, "%sC", &motorflag);
	asiString(chbuf);
	return err_prompt_detection();    			/* Check for return error value */
}	
    
        
int StepOpen(char motorflag, int stepsize)	
{
	if(stepsize <= 0 || stepsize >= 30000)  	/* Check to make sure step size is valid */
		return 128;    
	sprintf(chbuf, "%sIO %d", &motorflag, stepsize);
	asiString(chbuf);
	return err_prompt_detection();          	/* Check for return error value */
}
 
int StepClose(char motorflag, int stepsize) 
{   
	if(stepsize <= 0 || stepsize >= 30000)  	/* Check to make sure step size is valid */
		return 128;
	sprintf(chbuf, "%sIC %d", &motorflag, stepsize);
	asiString(chbuf);	
	return err_prompt_detection();          	/* Check for return error value */
}   

int GoToPosition(char motorflag, int position) 
{
	if(position < 0 || position > 30000)  		/* Check to make sure position is valid */
		return 128;          
	sprintf(chbuf, "%sM %d", &motorflag, position);	
	asiString(chbuf);
	return err_prompt_detection();          	/* Check for return error value */
}

int GoToDifferentPositions(int pos1, int pos2, int pos3, int pos4)
{												/* Goes to different positions simultaneously */
	int err, current_err=0;
	if(pos1<0||pos1>30000||pos2<0||pos2>30000||pos3<0||pos3>30000||pos4<0||pos4>30000)
		return 128;                             /* Check to make sure positions are valid */                                      
	sprintf(chbuf, "1FSET DP %d", pos1);		/* Set positions in parameters */	
	asiString(chbuf);	                        
 	flushSerial();  
 	err = err_prompt_detection();
	sprintf(chbuf, "2FSET DP %d", pos2);
	asiString(chbuf); 
	flushSerial();  
 	current_err = err_prompt_detection();
 	if (err == current_err);
 	else err = err + current_err;
	sprintf(chbuf, "3FSET DP %d", pos3);
	asiString(chbuf); 
	flushSerial();  
 	current_err = err_prompt_detection();
 	if (err == current_err);
 	else err = err + current_err;
	sprintf(chbuf, "4FSET DP %d", pos4);
	asiString(chbuf); 
	flushSerial();  
 	current_err = err_prompt_detection();
 	if (err == current_err);
 	else err = err + current_err;
	asiString("M");
	flushSerial(); 
 	current_err = err_prompt_detection();
 	if (err == current_err);
 	else err = err + current_err;				/* Accumulate all the errors */
 	return err;
}

int GoToHome()
{
	int err, current_err;    
	flushSerial();
	asiString("GO");							/* GraspOpen */
	err = err_prompt_detection();
	delay(&delay1); 
	flushSerial();
	asiString("SO");							/* SpreadOpen */   
	current_err = err_prompt_detection();
	if (err == current_err);
	else err = err + current_err;
	delay(&delay1);
	flushSerial();
	asiString("GM 0");							/* Get to position 0 for grasp */
	flushSerial();  
	current_err = err_prompt_detection();
	if (err == current_err);
	else err = err + current_err;
	delay(&delay1);
	flushSerial();
	asiString("SM 0");							/* Get to position 0 for spread */
	flushSerial(); 
	current_err = err_prompt_detection();
	if (err == current_err);
	else err = err + current_err;				/* Accumulate all the errors */
	return err;
}
	

int StopMotor(char motorflag, int pe)    				
{                   
    if (pe==1){
	   	flushSerial();
	    asiString("FSET PE 25");
	}
	sprintf(chbuf, "%sT", &motorflag);
	asiString(chbuf);
	return err_prompt_detection();    			/* Check for return error value */
}	                  

int Set(char motorflag, char parameter_flag, int value)
{
	if(value <0 || value>255)            		/* Check for value range */
		return 128;     
	if(parameter_flag == 'C')               	/* Convert parameter flags to Direct language */
		sprintf(smbuf, "CV");
	else if(parameter_flag == 'O')
		sprintf(smbuf, "OV");
	else if(parameter_flag == 'T')
		sprintf(smbuf, "PG");
	else if(parameter_flag == 'S')
		sprintf(smbuf, "MG"); 
	else if(parameter_flag == 'M')        		/* Motor on/off status*/
		sprintf(smbuf, "HOLD");					    /* On if 1, off if 0 */
	else if(parameter_flag == 'I')          	/* Hidden way to set PID */
		sprintf(smbuf, "IP"); 					    /* P is T */
	else if(parameter_flag == 'D')
		sprintf(smbuf, "DZ");
	else 
		return 64;
	sprintf(chbuf, "%sFSET %s %d", &motorflag, smbuf, value);
	asiString(chbuf);
	return err_prompt_detection();    			/* Check for return error value */
}	             

int Get(char motorflag, char parameter_flag, int *value)
{   
 int i, j;   
	if(parameter_flag == 'C')					/* Convert parameter flags to Direct language */
		sprintf(smbuf, "CV");
	else if(parameter_flag == 'O')
		sprintf(smbuf, "OV");
	else if(parameter_flag == 'T')
		sprintf(smbuf, "PG");
	else if(parameter_flag == 'P')
		sprintf(smbuf, "CP");
	else if(parameter_flag == 'S')
		sprintf(smbuf, "CG"); 
	else if(parameter_flag == 'M')  			/* Motor on/off status */
		sprintf(smbuf, "HOLD");
	else if(parameter_flag == 'I')				/* Hidden way to control PID */
		sprintf(smbuf, "IP");			 		   /* P is T */
	else if(parameter_flag == 'D')
		sprintf(smbuf, "DZ");
	else 
		return 64;
	sprintf(chbuf, "%sFGET %s", &motorflag, smbuf);
	asiString(chbuf);
	waitforAline(chbuf);						/* Get the value and convert from ASCII to decimal */
	waitforAline(chbuf);						/* For GraspGet, it will only get Finger1 value */
    value[0]=0;
	for (i=0;(chbuf[i]>=48)&&(chbuf[i]<=57);i++) {
		value[0]=(value[0] * 10) + (chbuf[i]-48);
		}
	if(motorflag=='G') { 
		i++;
		for(;(chbuf[i]>=48)&&(chbuf[i]<=57);i++) {
			value[1]=(value[1] * 10) + (chbuf[i]-48);
			}
		i++;
		for(;(chbuf[i]>=48)&&(chbuf[i]<=57);i++) {
			value[2]=(value[2] * 10) + (chbuf[i]-48);
			}
	}
	
	return err_prompt_detection();

}	             

                                    
int Save(char motorflag)
{                     
	sprintf(chbuf, "%sFSAVE", &motorflag);
	asiString(chbuf);
	return err_prompt_detection();     			/* Check for return error value */
}		     	
       
int Load(char motorflag)
{  
	sprintf(chbuf, "%sFLOAD", &motorflag);
	asiString(chbuf);
	return err_prompt_detection();     			/* Check for return error value */
}		     	   

int Default(char motorflag)
{      
	sprintf(chbuf, "%sFDEF", &motorflag);
	asiString(chbuf);
	return err_prompt_detection();     			/* Check for return error value */
}		     	    

int Temperature(int *temp)
{  
  int i;     
	*temp = 0;
	flushSerial();
	asiString("PGET TMP");
	waitforAline(chbuf);
	waitforAline(chbuf);						/* Convert temp from ASCII to digital */
	for(i=0;(chbuf[i]>=48)&&(chbuf[i]<=57);i++)	/* The temp is recorded in 1/10 degree */
		*temp = (*temp * 10) + (chbuf[i]-48);
	return err_prompt_detection();				/* Check for return error value */
}

int InitSoftware()
{
	asiString("RESET");
	return err_prompt_detection();				/* Check for return error value */
}

int InitHand()         						
{      
	asiString("FSET PE 1000");                 /*  Mask small resolution encoder errors  */
	flushSerial();
//    asiString("SM 600");
//	flushSerial();
	asiString("HI");    
	return err_prompt_detection();				/* Check for return error value */
} 


