
/*====================================================================*
 * TRDAQ.C                                                            *
 * Program to acquire motor torque ripple data.  Outputs binary data  *
 * to two separate files:                                             *
 *                                                                    *
 * raw_rip.dX  	  	First column is data collected from forward       *
 *					direction motor travel in forward direction;      *
 *					second column in reverse.  All zeroes are         *
 *					replaced by next nonzero entry.                   *
 *                                                                    *
 * smth_rip.dX		Filtered data from above.  Forward and reverse    *
 *					data for each position is averaged, then an       *
 *					11-point unity gain, bidirectional, averaging     *
 *					filter is applied.               				  *
 *                                                                    *
 * Call program with                                                  *
 *                                                                    *
 * trdaq X                                                         	  *
 *                                                                    *
 * where 	X is the axis desired (0-6)                               *
 *                                                                    *
 *                  1997 Barrett Technology, Inc.                     *
 *                          Karl Leeser                               *
 *====================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <graph.h>
#include <conio.h>
#include <float.h>
#include <math.h>
#include <sys\types.h>
#include <sys\timeb.h>
#include <string.h>  
#include "btech.h"
#include "pcdsp.h"


#define ACCURATE_WINDOW 50  /* Possibly tighten this */

int *smth_tr_ary;
int temp_tr_ary[4096][2];
int comp_tr_ary[4096]; 

/*----------------------temporary function prototypes-------------*/
char sample(int, int, int);
float set_pos(int, float);
/*----------------------------------------------------------------*/

void main(int argc, char *argv[])
{     
    /*int fp[10]={6000,20,19000,0,0,32767,0,32767,-5, 0};*/ /* arm motor gains*/

	int fp[10]={500,2,4000,0,0,32767,0,32767,-5,0};  /*wrist differential gains*/

	/*int fp[10]={900,0,7000,0,0,32767,0,32767,-5,0}; */ /*wrist roll gains*/

	int axis,pos;
	int counter;
	long count;
	struct videoconfig old_mode;  
	
	int key, i;
	char* testdatatype="smoothed";
	
	FILE *smth_tr_data_file;
	FILE *comp_tr_data_file;
	/* file that will be applied */
	char smth_filename[12] = "smth_tr.d";
	/* file that will be created */
	char comp_filename[12] = "comp_tr.d";
    	
	if ( argc > 1){
		if(sscanf(argv[1], "%d", &axis) <1 ){
			printf("Call with trdaq [0-6]\n");
			printf("where the number indicates\n");
			printf("the motor axis.\n");
			exit(0);
		}                                                   
	}
	else{
		printf("Call with trdaq [0-6]\n");
		printf("where the number indicates\n");
		printf("the motor axis.\n");
		exit(0);
	}
	
	
	strcat(smth_filename, argv[1]);
	strcat(comp_filename, argv[1]);
	
	/* load torque ripple file */
   	smth_tr_ary = (int*)calloc(TR_ARY_LENGTH,sizeof(int)); 
   	smth_tr_data_file = fopen(smth_filename, "rb");
	fflush(stdin);
	_makeTorqueRippleAry(&smth_tr_ary, smth_tr_data_file, smth_filename); 
//   	for (i=0; i<4096; i++)	printf("data [%d] = %d\n", i, smth_tr_ary[i]);
   	
	_clearscreen( _GCLEARSCREEN );
	fflush( stdin);
		
	system("cls");    
    printf("\t+-----------------------------------------------------------+\n"   
           "\t|                                                           |\n"
           "\t|       Torque ripple data acquisition program.             |\n"
           "\t|                                                           |\n"
           "\t|        Copywrite Barrett Technology, Inc. 1993,1997       |\n"
           "\t|                                                           |\n" 
           "\t+-----------------------------------------------------------+\n\n\n\n\n");
           
   initRobot();
	        
    set_in_position(axis,INDEX_WINDOW);
	set_filter(axis,fp);
   

	/* open compensated file */    
    if ( ( comp_tr_data_file= fopen( comp_filename, "wb" ) ) == NULL ){
		printf( "Error opening file comp_tr.d in this directory.\n" );
		exit( 1 );
	}
    
    /* Fill the arrays full of zeroes */
	for( counter = 0; counter <= 4096; counter++)
	{
		temp_tr_ary[counter][0]=0;
		temp_tr_ary[counter][1]=0;
	}

	_getvideoconfig( &old_mode );
	_setvideomode( _VRES16COLOR );

	 _clearscreen( _GVIEWPORT );
	 printf( "Press any key when we are ready to take the motor's data\n");
	 getch();
	 
	 set_position(axis, 0); 
        
	 printf( "Finding the index position for motor %d...\n\n", axis );
	 if (MarkerFound(axis)){
	 	/* Immediately zero the axis */
	 	/*printf("\nZeroing axis %d position.\n",axis);  
	 	set_position(axis, 0);  */
		printf( "\n\aFound! Press any key to continue\n");
		 getch();
	
	 }  
	 else{
		_setvideomode( old_mode.mode );
		printf(" Data taking aborted\n");
		fclose( smth_tr_data_file );
		fclose( comp_tr_data_file);
		deinitRobot();
		exit(1);
	 }  
  
     set_filter(axis,fp);   
    /* Refine the positioning accuracy on axis=axis */
    set_in_position(axis,ACCURATE_WINDOW);
    
	 
	 _clearscreen( _GVIEWPORT );
	 _setwindow( TRUE, -0.5, -1.0, 6.80, 2 );
	 _setcolor( White );
	 _rectangle_w( _GBORDER, 0, 2, 6.30, -1.0 );
	 _moveto_w( 0.0, 0 );
	 _lineto_w( 6.30, 0 );
	 _settextposition( 2,8 );
	 printf( "Plotting motor %d torque ripple data:", axis );

     /* Begin data acquisition */
	 _setcolor( Red );
	 if (sample(0, 4096, axis ) )
	 {
	  _setvideomode( old_mode.mode );
	  deinitRobot();
	  printf( "Keyboard Interrupt.  Data Corrupt\n" );
	  fclose( smth_tr_data_file ); 
	  fclose( comp_tr_data_file );
	  exit( 1 );
	 }

	 _setcolor( Blue );
	 if ( sample( 4096, 0, axis ) )
	 {
	  _setvideomode( old_mode.mode );
	  deinitRobot();
	  printf( "Keyboard Interrupt.  Data Corrupt\n" );
	  fclose( smth_tr_data_file );
	  fclose( comp_tr_data_file );
	  exit( 1 );
	 }

	
	/*******************************************************
	 * This next section replaces any zeroes in the array  *
	 * with the next nonzero entry.                        *
     * If the data for the index 'pos' is zero, use        *
     * the data in the first nonzero index afterwards.     *
     * If the end of the array is reached, wrap around.    * 
     *******************************************************/ 
	for(pos=0;pos<4096;pos++){
		count=0;	
		if(temp_tr_ary[pos+count][0]==0){   /* fix forward data */
			while(temp_tr_ary[pos+count][0]==0){
				if(pos+count>4096) count-=4096;
				count++;
				}
			temp_tr_ary[pos][0]=temp_tr_ary[pos+count][0];	
			}
			
				
		count=0;	
		if(temp_tr_ary[pos+count][1]==0){   /* fix reverse data */
			while(temp_tr_ary[pos+count][1]==0){
				if(pos+count>4096) count-=4096;
				count++;
				}
			temp_tr_ary[pos][1]=temp_tr_ary[pos+count][1];
			}
					
		}
	 
	/* create single array of averaged forward/reverse values *
     * and print new values for each index */
    //printf("Type 'y' to observe averaged data, any other key otherwise.\n");
    //if(getch()=='y') show=TRUE;
//    for(pos=0;pos<4096;pos++){
    	/* made new array (rough) to average + and - raw data values so that the resulting
           file would be compatable with tr_test 980313 JG*/
//    	comp_tr_ary[pos]=(temp_tr_ary[pos][0]+temp_tr_ary[pos][1])/2;
    	//if(show) printf("%d \t%f\n",pos,smth_tr_ary[pos]);
//    	}
    
	/* Write compensated data to file */
	fwrite( temp_tr_ary, sizeof(temp_tr_ary), 1, comp_tr_data_file );
	fclose( comp_tr_data_file );
	
	    
    _settextposition(4, 8);
	printf("Data collection complete.  Press any key to continue.");
    getch();
	_setvideomode( old_mode.mode );
	
	controller_idle(axis);
	

	printf("Now test the motor shaft to see if the inverse plant is working.\n");
	printf("Press any key to quit the program.\n");
	
	/* ===============================Test loop===================*/
	while ( !kbhit() )
	{ 
		pos=getAutoRangedPosition(axis);
		setMotorTorque(axis, smth_tr_ary[pos]);
	}   
	
    printf("\n\n\n");
	
	deinitRobot();
	}

/*============================================================================*
 * FUNCTION DEFINITIONS                                                       *
 *============================================================================*/
char sample( int begin, int end, int axis )
{
	   int real_pos, index;
	   int direction;
	   char in;
	   int position;
	   float sum;
	   int count;            
	   float pixely;
    	    	
		
		/* figure out whether we're going forwards or backwards */
	   if (end>begin){
		direction = 1;
	   }
	   else{
		direction = -1;
	   }

		/* now figure out which of the data arrays to write to */
	   if (direction == 1){
		index = 0;
	   }
	   else{
		index = 1;
	   }

	   /*t[axis] [direction]=0;*/
	   
	   in = 0;
	    /* now take data*/
	   for( position=begin; !(position == end); position += direction  )
	   {
			if ( kbhit() )
			{
			 in = getch();
			 break;
			}

			/* position/((end-begin)/2pi)*/
			set_pos( axis, (float)position / (float)CONVERT  );
			sum=0;
        
			for( count=0; count < SAMPLES; count++ )
			{
			  sum=getMotorTorque(axis) + sum;
			}
            
        							
			real_pos=getAutoRangedPosition(axis);
			pixely = sum / SAMPLES - smth_tr_ary[real_pos];    
			temp_tr_ary[real_pos] [index] = (int)(sum / SAMPLES) - smth_tr_ary[real_pos];
			_setpixel_w( ((real_pos*2*PI)/4096.0), pixely/SCALE);
	   }
	   return ( in );
}


float set_pos(int axis, float position){
	int torque;
	static int on[4] = {0,0,0,0};
	struct _rccoord oldpos;
		
	if (on[axis] == 0){
		on[axis] = 1;
		controller_run(axis);
	}               
	start_move( axis, position * (float)(4096/ (2 * PI)), SLOW_VEL, SLOW_ACCEL);

	while(!axis_done(axis)){
		oldpos = _gettextposition();
		_settextposition( 29, 8);
		printf("Position of axis: %5.2f   \r", position*CONVERT);
	}
	_settextposition(oldpos.row, oldpos.col);
	get_dac_output(axis, &torque);
	return((float)torque);
} 



