/*====================================================================*
 * TRTEST.C                                                           *
 * Tests torque ripple data.  Now tests only new integer data.        *
 *                  1997 Barrett Technology, Inc.                     *
 *                          DR, Karl Leeser                           *
 *====================================================================*/                                                       

#include <time.h>
#include <graph.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <stdio.h>
#include <sys\types.h>
#include <sys\timeb.h>
#include "pcdsp.h"
#include <direct.h>  
#include <string.h>

#include "btech.h"

/*  added for automatically finding the marker pulse */
#define INDEX_WINDOW 50
#define ACCURATE_WINDOW 5


#define ARY_LENGTH 4096

/* acceleration and velocity for move command*/
#define VEL 10000
#define ACCEL 100000

/* added to move links back to zero position slowly */
//#define VEL_SLOW 1000
//#define ACCEL_SLOW 10000



void main(int argc, char *argv[]){   
    int axis, test,i;   
	int pos;
	int *rip_ary ;
	int option = TRUE;	
	FILE *ripple_data;
	double dummy;
	float comp;
	
	char filenames[12] = "smth_tr.d";
	char filenamea[12] = "avrg_tr.d";
	char answer, key;
	char* testdatatype="compensated";
	
	int fp[10]={0,0,0,0,0,0,0,0,0,0};
	int fpdefault[10]={3000,8,12000,0,0,32767,0,32767,-5,0};   
   
   	
   		printf("\t+-----------------------------------------------------------+\n"
	       "\t|                                                           |\n"
	       "\t|                                                           |\n"
	       "\t|           Welcome to a torque ripple eliminator.          |\n"
	       "\t|       Copywrite Barrett Technology, Inc. 1993, 1997, 1998 |\n"
	       "\t|                                                           |\n"
           "\t|                                                           |\n"
           "\t+-----------------------------------------------------------+\n\n\n\n");
 
   	
   	rip_ary = (int*)calloc(TR_ARY_LENGTH,sizeof(int)); 
   	
                                                           

   	if ( argc > 1)
   	{
		if(sscanf(argv[1], "%d", &axis) <1 )
		{
			printf("Call with TRTEST [0-6]\n");
			printf("where the number indicates\n");
			printf("the motor axis.\n");
			exit(0);
		}                                                   
	}
	
	else
	{
		printf("Call with TRTEST [0-6]\n");
		printf("where the number indicates\n");
		printf("the motor axis.\n");
		exit(0);
	}
	
	initRobot();
    
    /* prints parameters and give option to change */
    printf("\nThe default Parameters are:");
	printf("\nProportional Gain: %d", fpdefault[0]);
	printf("\nIntegral Gain: %d", fpdefault[1]);
	printf("\nDerivative Gain: %d", fpdefault[2]); 
	printf("\nWould you like to change the gains?   y/n:   ");
	scanf("%c", &answer);
	if (answer == 'y' || answer == 'Y' )
	{
		printf("\nProportional Gain = ");
		scanf("%d",&fpdefault[0]);
		printf("\nIntegral Gain = ");
		scanf("%d",&fpdefault[1]);
		printf("\nDerivative Gain = ");
		scanf("%d",&fpdefault[2]);
	}
	for (i=0;i<10;i++)
    		fp[i]=fpdefault[i];	     
        
    set_filter(axis,fp);
	printf("\nThe gain parameters will be:");
	printf("\nProportional Gain: %d", fp[0]);
	printf("\nIntegral Gain: %d", fp[1]);
	printf("\nDerivative Gain: %d", fp[2]); 

	while(option)
	{
		printf("\n\nSmooth data press <1>");
		printf("\nAverage data press <2>");
		scanf("%d", &test);
	
		switch (test)
		{	
			case 1:	
				strcat(filenames, argv[1]);
		        printf("\nopening file %s to read in filtered data", filenames);
				if ( ( ripple_data = fopen( filenames, "rb" ) ) == NULL )
				{
					printf( "Error opening file in this directory.\n" );
					exit( 1 );
				}
		        fflush(stdin);
    			_makeTorqueRippleAry(&rip_ary, ripple_data, filenames); 
            	option=FALSE;
            	break;
    	    
    	    case 2:	
				strcat(filenamea, argv[1]);
		        printf("\nopening file %s to read in averaged data", filenamea);
				if ( ( ripple_data = fopen( filenamea, "rb" ) ) == NULL )
				{
					printf( "Error opening file in this directory.\n" );
					exit( 1 );
				}
		        fflush(stdin);
    			_makeTorqueRippleAry(&rip_ary, ripple_data, filenamea); 
            	option=FALSE;
            	break;
    	    
			default:
				printf("invalid key press 1 or 2 please!");
				break;
			
		}
	}


	system("cls");
	set_position(axis,0);
    printf("Enable the Robot and then press any key");
    getch();


    /*=================Find index markers===================*/
       	
       	set_in_position(axis,INDEX_WINDOW);
       	printf( "\n\nFinding the index position for motor %d...\n", axis );
		if (MarkerFound(axis)){
			printf("Marker pulse found!.\n");
			}
		else{
			printf(" Data taking aborted\n");
            goto end;
			}
	
	/* idles the motors */
	controller_idle(axis);
	
	printf("Prepare to test torque ripple compensation on axis %d.\n",axis);
	printf("Press any key when ready.\n");
	getch();


	system("cls");    
    printf("\t+-----------------------------------------------------------+\n"
	       "\t|                                                           |\n"
	       "\t|                                                           |\n"
	       "\t|  Test torque ripple compensation.  Press <space> to exit. |\n"
	       "\t|                                                           |\n"
	       "\t|                                                           |\n"
           "\t+-----------------------------------------------------------+");
    
	key = 'c';
	printf("\nPress <c> to apply compensated data.");
	printf("\nPress <n> for no compensation\n");
	
	/* ===============================Test loop===================*/
	while ( key != ' ' )
	{ 
		printf("Sending %s data to axis %d.                            \r",testdatatype, axis);
		pos=getAutoRangedPosition(axis);
		
		if( kbhit()) key = getch(); 
		switch(key){
			case 'c':  
				comp=rip_ary[pos];   /* Average the data for both directions */ 
				testdatatype="compensated";
				break;	
			case 'n':default:
				comp=0;
				testdatatype="no";
				break;
		} 
		if(pos>4095 || pos<0) printf("\nRange error! pos=%d\n",pos);
		setMotorTorque(axis, (int)comp);
    }
    /*--------------------End of Primary loop----------------------*/
    end:
    deinitRobot();
                
	fclose(ripple_data);
  	
  	free(rip_ary);
  	
    printf("\nExiting trtest.\n" );  
	
}

